@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Variables that are needed globally, and even by the very basic libs.
  *
@@ -368,6 +368,14 @@ after_dot:
 					{
 						requested_width = requested_precision;
 					}
+					if (requested_precision > width)
+					{
+						requested_precision = width;
+					}
+					if (requested_width > width)
+					{
+						requested_width = width;
+					}
 					if (requested_precision > requested_width)
 					{
 						requested_precision = requested_width;
@@ -408,6 +416,14 @@ after_dot:
 					{
 						requested_width = requested_precision;
 					}
+					if (requested_precision > width)
+					{
+						requested_precision = width;
+					}
+					if (requested_width > width)
+					{
+						requested_width = width;
+					}
 					if (requested_precision > requested_width)
 					{
 						requested_precision = requested_width;
@@ -508,6 +524,14 @@ after_dot:
 								{
 									requested_width = requested_precision;
 								}
+								if (requested_precision > width)
+								{
+									requested_precision = width;
+								}
+								if (requested_width > width)
+								{
+									requested_width = width;
+								}
 								if (requested_precision > requested_width)
 								{
 									requested_precision = requested_width;
@@ -582,6 +606,14 @@ after_dot:
 								{
 									requested_width = requested_precision;
 								}
+								if (requested_precision > width)
+								{
+									requested_precision = width;
+								}
+								if (requested_width > width)
+								{
+									requested_width = width;
+								}
 								if (requested_precision > requested_width)
 								{
 									requested_precision = requested_width;
@@ -656,6 +688,37 @@ after_dot:
 	va_end(ap);
 }
 
+static void display_frame (uint16_t y, uint16_t x, uint16_t h, uint16_t w, uint8_t color, const char *title, uint16_t dot, uint16_t hbar1, uint16_t hbar2)
+{
+	size_t len = strlen (title);
+	int left;
+	int right;
+
+	if (len > (w - 4))
+	{
+		len = w - 4;
+	}
+	left = (w - 4 - len) / 2;
+	right = w - 4 - len - left;
+
+	/* dot = 0, no dot, otherwize it says which line number after Y to place it at */
+	display_nprintf (y++, x, color, w, "\xda%*C\xc4 %*s %*C\xc4\xbf", left, len, title, right); h--; dot--; hbar1--; hbar2--;
+	while (h > 1)
+	{
+		if ((!hbar1) || (!hbar2))
+		{
+			display_nprintf (y++, x, color, w, "\xc3%*C\xc4\xb4", w - 2);
+		} else {
+			display_nprintf (y++, x, color, w, "\xb3%*C %c", w - 2, dot ? '\xb3': '\xdd');
+		}
+		h--;
+		dot--;
+		hbar1--;
+		hbar2--;
+	}
+	display_nprintf (y++, x, color, w, "\xc0%*C\xc4\xd9", w - 2); h--;
+}
+
 static void writenum (uint16_t *buf, uint16_t ofs, uint8_t attr, unsigned long num, uint8_t radix, uint16_t len, int clip0)
 {
 	char convbuf[20];
@@ -701,12 +764,14 @@ struct console_t Console =
 {
 	&dummyConsoleDriver,
 	display_nprintf,
+	display_frame,
 	writenum,
 	writestring,
 	writestringattr,
 	ekbhit,
 	egetch,
 	framelock,
+	poll_framelock,
 #ifdef HAVE_LZW
 	GIF87_try_open_bgra,
 #else
@@ -716,6 +781,7 @@ struct console_t Console =
 	try_open_png,
 	EditStringUTF8z,
 	EditStringUTF8,
+	EditStringASCII,
 	80,      /* TextHeight */
 	25,      /* TextWidth */
 	0,       /* TextGUIOverlay */
