@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * The little program that loads up the basic libs and starts main. It can also
  * start up the gdb debugger on fatal signals.
@@ -382,7 +382,7 @@ static char *validate_xdg_dir_absolute (const char *name, const char *def)
 	}
 	if (xdg)
 	{
-		char *retval = malloc (strlen (xdg) + 5);
+		char *retval = malloc (strlen (xdg) + 5 + 1);
 		if (retval)
 		{
 			sprintf (retval, "%s%socp/",
@@ -783,7 +783,7 @@ int validate_home(void)
 		char settingsPath[PATH_MAX];
 		if (find_directory(B_USER_SETTINGS_DIRECTORY, -1, false, settingsPath, sizeof(settingsPath)) == B_OK)
 		{
-			_cfConfigHomeDir = malloc (strlen (settingsPath) + 5);
+			_cfConfigHomeDir = malloc (strlen (settingsPath) + 5 + 1);
 			if (!_cfConfigHomeDir)
 			{
 				fprintf (stderr, "malloc() failed\n");
