@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * PMain - main module (loads and inits all startup modules)
  *
@@ -121,7 +121,9 @@ static int init_modules(int argc, char *argv[])
 	if ((ret=cmdhlp()))
 		return ret;
 
+#ifndef _WIN32
 	if (!geteuid())
+	{
 		if (getuid())
 		{
 			fprintf(stderr, "Changing user to non-root\n");
@@ -131,7 +133,9 @@ static int init_modules(int argc, char *argv[])
 				return errGen;
 			}
 		}
+	}
 	if (!getegid())
+	{
 		if (getgid())
 		{
 			fprintf(stderr, "Changing group to non-root\n");
@@ -141,6 +145,8 @@ static int init_modules(int argc, char *argv[])
 				return errGen;
 			}
 		}
+	}
+#endif
 
 	framelock_init ();
 
@@ -785,9 +791,9 @@ static int init_modules(int argc, char *argv[])
 		{
 			char *kernal, *basic, *chargen, *iter;
 			printf("ocp.ini update (0.2.95) libsidplayfp ROM paths changed into UNIX syntax\n");
-			kernal  = strdup (cfGetProfileString ("libsidplay", "kernal",  "KERNAL.ROM"));
-			basic   = strdup (cfGetProfileString ("libsidplay", "basic",   "BASIC.ROM"));
-			chargen = strdup (cfGetProfileString ("libsidplay", "chargen", "CHARGEN.ROM"));
+			kernal  = strdup (cfGetProfileString ("libsidplayfp", "kernal",  "KERNAL.ROM"));
+			basic   = strdup (cfGetProfileString ("libsidplayfp", "basic",   "BASIC.ROM"));
+			chargen = strdup (cfGetProfileString ("libsidplayfp", "chargen", "CHARGEN.ROM"));
 			for (iter = kernal; *iter; iter++)
 			{
 				if (*iter == '/')
@@ -818,9 +824,9 @@ static int init_modules(int argc, char *argv[])
 					*iter = '/';
 				}
 			}
-			cfSetProfileString ("libsidplay", "kernal",  kernal);
-			cfSetProfileString ("libsidplay", "basic",   basic);
-			cfSetProfileString ("libsidplay", "chargen", chargen);
+			cfSetProfileString ("libsidplayfp", "kernal",  kernal);
+			cfSetProfileString ("libsidplayfp", "basic",   basic);
+			cfSetProfileString ("libsidplayfp", "chargen", chargen);
 			free (kernal);
 			free (basic);
 			free (chargen);
@@ -869,28 +875,190 @@ static int init_modules(int argc, char *argv[])
 			cfRemoveEntry("defaultconfig", "link");
 		}
 
-		if (epoch < 20220919)
+		if (epoch < 20221228)
 		{
-			cfSetProfileInt("version", "epoch", 20220919, 10);
+			char *tmp, *iter;
+
+			fprintf(stderr, "ocp.ini update (0.2.102) removes postprocadds\n");
+			cfRemoveEntry("devwMix", "postprocadds");
+			cfRemoveEntry("devwMixQ", "postprocadds");
+			cfRemoveEntry("devwMixF", "postprocadds");
+
+			fprintf(stderr, "ocp.ini update (0.2.102) _iReverb and _fReverb should be named iReverb and fReverb\n");
+
+			if (!strcmp (cfGetProfileString ("devwMix", "postprocs", ""),  "_iReverb")) cfSetProfileString ("devwMix",  "postprocs", "iReverb");
+			if (!strcmp (cfGetProfileString ("devwMixQ", "postprocs", ""), "_iReverb")) cfSetProfileString ("devwMixQ", "postprocs", "iReverb");
+			if (!strcmp (cfGetProfileString ("devwMixF", "postprocs", ""), "_fReverb")) cfSetProfileString ("devwMixF", "postprocs", "fReverb");
+
+			/* We silently repair the case-error in devwmixQ, this has no effect other than looking more consistent */
+			tmp = strdup (cfGetProfileString ("sound", "wavetabledevices", ""));
+			iter = strstr (tmp, "devwmixQ");
+			if (iter)
+			{
+				iter[4] = 'M';
+				cfSetProfileString ("sound", "wavetabledevices", tmp);
+			}
+			free (tmp);
+		}
+
+		if (epoch < 20230211)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpALSA] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpCA]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpDisk] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpMPx]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpNone]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpOSS] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpSDL]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpSDL2]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwNone]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMix] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMix] subtype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMixQ] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMixQ] subtype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMixF] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMixF] mixResample=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) defaults [devpALSA] card=default (former name was path=)\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) defaults [devpALSA] mixer=default\n");
+
+			cfRemoveEntry  ("devpALSA", "link");
+			cfRemoveProfile("devpCA"  );
+			cfRemoveEntry  ("devpDisk", "link");
+			cfRemoveProfile("devpMPx" );
+			cfRemoveProfile("devpNone");
+			cfRemoveEntry  ("devpOSS" , "link");
+			cfRemoveProfile("devpSDL" );
+			cfRemoveProfile("devpSDL2");
+			cfRemoveProfile("devwNone");
+			cfRemoveEntry  ("devwMix" , "link");
+			cfRemoveEntry  ("devwMix" , "subtype");
+			cfRemoveEntry  ("devwMixQ", "link");
+			cfRemoveEntry  ("devwMixQ", "subtype");
+			cfRemoveEntry  ("devwMixF", "link");
+			cfRemoveEntry  ("devwMixF", "mixResample");
+			cfSetProfileString ("devpALSA", "card", cfGetProfileString ("devpALSA", "path", "default"));
+			cfRemoveEntry  ("devpALSA", "path");
+			cfSetProfileString ("devpALSA", "mixer", cfGetProfileString ("devpALSA", "mixer", "default"));
+		}
+
+		if (epoch < 20230213)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.103) adds comment on [screen] mvoltype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) adds comment on [screen] insttype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) adds comment on [screen] channeltype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) adjust the default [screen] screentype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) replaces comment on [screen] screentype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) move [x11] font= to [screen] fontsize=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) add [screen] winwidth=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) add [screen] winheight=\n");
+
+			cfSetProfileComment ("screen", "mvoltype", cfGetProfileComment ("screen", "mvoltype", "; 0=none, 1=big, 2=side (only in >132 column modes)"));
+			cfSetProfileComment ("screen", "insttype", cfGetProfileComment ("screen", "insttype", "; 0=none, 1=short, 2=long, 3=side (only in >132 column modes)"));
+			cfSetProfileComment ("screen", "channeltype", cfGetProfileComment ("screen", "channeltype", "; 0=none, 1=short, 2=long, 3=side (only in >132 column modes)"));
+			cfSetProfileInt     ("screen", "screentype", 5, 10);
+			cfSetProfileComment ("screen", "screentype", "; 0=80x25, 1=80x30, 2=80x50, 3=80x60, 4=132x25, 5=132x30, 6=132x50, 7=132x60, 8=custom");
+			cfSetProfileString  ("screen", "fontsize", cfGetProfileString ("x11", "font", "1"));
+			cfSetProfileComment ("screen", "fontsize", "; if screentype=8: 0=8x8, 1=8x16");
+			cfRemoveEntry       ("x11", "font");
+			cfSetProfileInt     ("screen", "winwidth", 1280, 10);
+			cfSetProfileComment ("screen", "winwidth", "; if screentype=8");
+			cfSetProfileInt     ("screen", "winheight", 1024, 10);
+			cfSetProfileComment ("screen", "winheight", "; if screentype=8");
+		}
+
+		if (epoch < 20230214)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.103), 0.2.91 update missed setting the color for XM files.\n");
+
+			cfSetProfileInt ("fscolors", "XM", cfGetProfileInt ("fscolors", "XM", 3, 10), 10);
+		}
+
+		if (epoch < 20230403)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.104), added [adplug] emulator=nuked\n");
+
+			cfSetProfileString  ("adplug", "emulator", "nuked");
+			cfSetProfileComment ("adplug", "emulator", "; Possible values are ken, nuked, satoh and woody.");
+		}
+
+		if (epoch < 20230607)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.106), added playgme plugin\n");
+			cfSetProfileInt ("fscolors", "AY2", 13, 10);
+			cfSetProfileInt ("fscolors", "GBS", 13, 10);
+			cfSetProfileInt ("fscolors", "GYM", 13, 10);
+			cfSetProfileInt ("fscolors", "HES", 13, 10);
+			cfSetProfileInt ("fscolors", "KSS", 13, 10);
+			cfSetProfileInt ("fscolors", "NSF", 13, 10);
+			cfSetProfileInt ("fscolors", "NSFe", 13, 10);
+			cfSetProfileInt ("fscolors", "SAP", 13, 10);
+			cfSetProfileInt ("fscolors", "SPC", 13, 10);
+			cfSetProfileInt ("fscolors", "VGM", 13, 10);
+		}
+
+		if (epoch < 20230630)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.106), added [adplug] retrowave support\n");
+			cfSetProfileComment ("adplug", "emulator", "; Possible values are ken, nuked, satoh, woody and retrowave.");
+			cfSetProfileString  ("adplug", "retrowave", "auto");
+			cfSetProfileComment ("adplug", "retrowave", "; Device to use, e.g. /dev/ttyACM0 /dev/cuaU0 /dev/dtyU0 /dev/cu.usbmodem0000001 COM1");
+		}
+
+		if (epoch < 20240107)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.107) add [fscolors] UNKN=7\n");
+			cfSetProfileInt     ("fscolors", "UNKN", 7, 10);
+		}
+
+		if (epoch < 20240510)
+		{
+			fprintf (stderr, "ocp.ini update (0.2.110) add [modland.com] mirror=https://ftp.modland.com\n");
+#ifdef _WIN32
+			fprintf (stderr, "ocp.ini update (0.2.110) add [modland.com] cachedir=$OCPDATAHOME\\modland.com\\\n");
+#else
+			fprintf (stderr, "ocp.ini update (0.2.110) add [modland.com] cachedir=$OCPDATAHOME/modland.com/\n");
+#endif
+			fprintf (stderr, "ocp.ini update (0.2.110) add [modland.com] showrelevantdirectoriesonly=1\n");
+
+			cfSetProfileString ("modland.com", "mirror", "https://ftp.modland.com");
+#ifdef _WIN32
+			cfSetProfileString ("modland.com", "cachedir", "$OCPDATAHOME\\modland.com\\");
+#else
+			cfSetProfileString ("modland.com", "cachedir", "$OCPDATAHOME/modland.com/");
+#endif
+			cfSetProfileBool ("modland.com", "showrelevantdirectoriesonly", 1);
+		}
+
+		if (epoch < 20240510)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.107) add [libsidplayfp] filterrange6581=0.5\n");
+			cfSetProfileString  ("libsidplayfp", "filterrange6581", "0.5");
+			fprintf(stderr, "ocp.ini update (0.2.107) add [libsidplayfp] combinedwaveforms=Strong\n");
+			cfSetProfileString  ("libsidplayfp", "combinedwaveforms", "Strong");
+		}
+
+		if (epoch < 20240510)
+		{
+			cfSetProfileInt("version", "epoch", 20240510, 10);
 			cfStoreConfig();
 			if (isatty(2))
 			{
-				fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing this:\033[0m\nrm -f ~/.ocp/ocp.ini\n\n");
+				fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing either:\033[0m\nrm -f ~/.ocp/ocp.ini\033[1m\033[31m or \033[0m\nrm -f $XDG_CONFIG_HOME/ocp/ocp.ini\n\n");
 			} else {
-				fprintf(stderr,"\nWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing this:\nrm -f ~/.ocp/ocp.ini\n\n");
+				fprintf(stderr,"\nWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing either:\nrm -f ~/.ocp/ocp.ini or rm -f $XDG_CONFIG_HOME/ocp/ocp.ini\n\n");
 			}
 			sleep(5);
 		}
 	}
-	if (cfGetProfileInt("version", "epoch", 0, 10) != 20220919)
+	if (cfGetProfileInt("version", "epoch", 0, 10) != 20240510)
 	{
 		if (isatty(2))
 		{
-			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20220919\033[0m\n\n");
+			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20240510\033[0m\n\n");
+			sleep(5);
 		} else {
-			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20220919\n\n");
+			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20240510\n\n");
 		}
-		sleep(5);
 	}
 
 	cfScreenSec=cfGetProfileString(cfConfigSec, "screensec", "screen");
@@ -899,9 +1067,9 @@ static int init_modules(int argc, char *argv[])
 	lnkLink(cfGetProfileString2(cfConfigSec, "defaultconfig", "prelink", ""));
 	lnkLink(cfGetProfileString("general", "prelink", ""));
 
-	if (lnkLinkDir(cfProgramDirAutoload)<0)
+	if (lnkLinkDir(cfProgramPathAutoload)<0)
 	{
-		fprintf(stderr, "could not autoload directory: %s\n", cfProgramDirAutoload);
+		fprintf(stderr, "could not autoload directory: %s\n", cfProgramPathAutoload);
 		return -1;
 	}
 
@@ -950,9 +1118,10 @@ int failcheck(signed int source, signed int filter)
 }
 #endif
 
-static int _bootup(int argc, char *argv[], const char *ConfigDir, const char *DataDir, const char *ProgramDir)
+static int _bootup(int argc, char *argv[], const char *HomePath, const char *ConfigHomePath, const char *ConfigDataPath, const char *DataPath, const char *ProgramPath)
 {
 	int result;
+#ifndef _WIN32
 	if (isatty(2))
 	{
 		fprintf(stderr, "\033[33m\033[1mOpen Cubic Player for Unix \033[32mv" VERSION "\033[33m, compiled on %s, %s\n", compiledate, compiletime);
@@ -979,20 +1148,31 @@ static int _bootup(int argc, char *argv[], const char *ConfigDir, const char *Da
 	}
 	fprintf(stderr, "pass\n");
 #endif
+#else
+	fprintf(stderr, "Open Cubic Player for mingw v" VERSION ", compiled on %s, %s\n", compiledate, compiletime);
+	fprintf(stderr, "Ported to Unix and mingw by Stian Skjelstad\n");
+#endif
 
-	cfConfigDir = (char *)ConfigDir;
-	cfDataDir = strdup (DataDir);
-	cfProgramDir = (char *)ProgramDir;
-	cfProgramDirAutoload = malloc (strlen (cfProgramDir) + 9 + 1);
-	sprintf(cfProgramDirAutoload, "%sautoload/", cfProgramDir);
+	cfHomePath = (char *)HomePath;
+	cfConfigHomePath = (char *)ConfigHomePath;
+	cfDataHomePath = (char*)ConfigDataPath;
+	cfDataPath = strdup (DataPath);
+	cfProgramPath = (char *)ProgramPath;
+	cfProgramPathAutoload = malloc (strlen (cfProgramPath) + 9 + 1);
+#ifdef _WIN32
+	sprintf(cfProgramPathAutoload, "%sautoload\\", cfProgramPath);
+#else
+	sprintf(cfProgramPathAutoload, "%sautoload/", cfProgramPath);
+#endif
 
 	if (cfGetConfig(argc, argv))
 	{
-		cfConfigDir = 0;
-		free (cfDataDir); cfDataDir = 0;
-		cfProgramDir = 0;
-		free (cfTempDir); cfTempDir = 0;
-		free (cfProgramDirAutoload); cfProgramDirAutoload = 0;
+		cfConfigHomePath = 0;
+		cfDataHomePath = 0;
+		free (cfDataPath); cfDataPath = 0;
+		free (cfTempPath); cfTempPath = 0;
+		cfProgramPath = 0;
+		free (cfProgramPathAutoload); cfProgramPathAutoload = 0;
 		return -1;
 	}
 
@@ -1005,11 +1185,12 @@ static int _bootup(int argc, char *argv[], const char *ConfigDir, const char *Da
 
 	cfCloseConfig();
 
-	cfConfigDir = 0;
-	free (cfDataDir); cfDataDir = 0;
-	cfProgramDir = 0;
-	free (cfTempDir); cfTempDir = 0;
-	free (cfProgramDirAutoload); cfProgramDirAutoload = 0;
+	cfConfigHomePath = 0;
+	cfDataHomePath = 0;
+	free (cfDataPath); cfDataPath = 0;
+	free (cfTempPath); cfTempPath = 0;
+	cfProgramPath = 0;
+	free (cfProgramPathAutoload); cfProgramPathAutoload = 0;
 
 
 	return 0;
