@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * OCP.INI file and environment reading functions
  *
@@ -103,7 +103,7 @@ static int readiniline(char *key, char *str, char *comment, const char *line)
 		eol++;
 	if ((eol[0]==';')||(eol[0]=='#'))
 	{
-		strncpy(comment, eol, COMMENTBUF_LEN);
+		strncpy(comment, eol, COMMENTBUF_LEN-1);
 		comment[COMMENTBUF_LEN-1]=0;
 	}
 	while (isspace(eol[-1]))
@@ -698,18 +698,31 @@ int cfGetConfig(int argc, char *argv[])
 		cfDataPath = strdup (t);
 	}
 
-	if ((t=_cfGetProfileString("general", "tempdir", t)))
+	t=_cfGetProfileString("general", "tempdir", 0);
+	if (!t || !strlen (t))
 	{
-		cfTempPath = strdup (t);
-	} else if ((t=getenv("TEMP")))
-	{
-		cfTempPath = strdup (t);
-	} else if ((t=getenv("TMP")))
-	{
-		cfTempPath = strdup (t);
-	} else {
-		cfTempPath = strdup ("/tmp/");
+		t=getenv("TEMP");
 	}
+	if (!t || !strlen (t))
+	{
+		t=getenv("TMP");
+	}
+	if (!t || !strlen (t))
+	{
+#ifdef _WIN32
+		t = "c:\\TEMP\\";
+#else
+		t = "/tmp/";
+#endif
+	}
+	cfTempPath = malloc (strlen (t) + 2);
+#ifdef _WIN32
+# define c "\\"
+#else
+# define c "/"
+#endif
+	sprintf (cfTempPath, "%s%s", t, (t[strlen(t) - 1] != c[0]) ? c : "");
+#undef c
 
 #ifdef PSETTING_DEBUG
 	{
