@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * CPIface link info screen
  *
@@ -163,7 +163,7 @@ static void hlpDraw (struct cpifaceSessionAPI_t *cpifaceSession)
 	plDisplayHelp (cpifaceSession);
 }
 
-static void hlpSetMode()
+static void hlpSetMode(struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	cpiSetTextMode(fsScrType);
 	plWinHeight=plScrHeight-6;
