@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * CPIFace song message mode
  *
@@ -111,7 +111,7 @@ static void msgDraw (struct cpifaceSessionAPI_t *cpifaceSession)
 	plDisplayMessage (cpifaceSession);
 }
 
-static void msgSetMode(void)
+static void msgSetMode(struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	cpiSetTextMode(0);
 	plWinFirstLine=6;
