@@ -1,5 +1,5 @@
 /** OpenCP Module Player
- * copyright (c) 2006-'21 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2006-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * CoreAudio (Darwin/Mac OS/OSX) Player device
  *
@@ -34,9 +34,10 @@
 #include "boot/plinkman.h"
 #include "boot/psetting.h"
 #include "cpiface/cpiface.h"
-#include "dev/imsdev.h"
+#include "dev/deviplay.h"
 #include "dev/player.h"
 #include "dev/ringbuffer.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
 
 #ifdef COREAUDIO_DEBUG
@@ -45,10 +46,12 @@
 # define debug_printf(...) do {} while(0)
 #endif
 
+static const struct plrDriverAPI_t *plrDriverAPI;
+
 static AudioUnit theOutputUnit;
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutexattr_t mta;
-struct sounddevice plrCoreAudio;
+static const struct plrDriver_t plrCoreAudio;
 static int needfini=0;
 
 static void *devpCoreAudioBuffer;
@@ -172,8 +175,8 @@ static OSStatus theRenderProc(void *inRefCon, AudioUnitRenderActionFlags *inActi
 		lastCallbackTime += tp.tv_nsec / 1000;
 	}
 
-	ringbuffer_get_tail_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
-	ringbuffer_tail_consume_samples (devpCoreAudioRingBuffer, length1 + length2);
+	plrDriverAPI->ringbufferAPI->get_tail_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
+	plrDriverAPI->ringbufferAPI->tail_consume_samples (devpCoreAudioRingBuffer, length1 + length2);
 
 	debug_printf ("theRenderProc: consumed %d + %d (paused=%d)\n", length1, length2, devpCoreAudioPauseSamples);
 
@@ -187,7 +190,7 @@ static OSStatus theRenderProc(void *inRefCon, AudioUnitRenderActionFlags *inActi
 		}
 	}
 
-	ringbuffer_get_processing_bytes (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
+	plrDriverAPI->ringbufferAPI->get_processing_bytes (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
 
 	debug_printf ("theRenderProc: processing available %d + %d, len %d => %d\n", length1, length2, len, (length1 > len) ? len : length1);
 
@@ -197,7 +200,7 @@ static OSStatus theRenderProc(void *inRefCon, AudioUnitRenderActionFlags *inActi
 	}
 
 	memcpy(stream, (uint8_t *)devpCoreAudioBuffer + pos1, length1);
-	ringbuffer_processing_consume_bytes (devpCoreAudioRingBuffer, length1);
+	plrDriverAPI->ringbufferAPI->processing_consume_bytes (devpCoreAudioRingBuffer, length1);
 	len -= length1;
 	stream += length1;
 	lastLength = length1 >> 2 /* stereo + bit16 */;
@@ -209,7 +212,7 @@ static OSStatus theRenderProc(void *inRefCon, AudioUnitRenderActionFlags *inActi
 			length2 = len;
 		}
 		memcpy (stream, (uint8_t *)devpCoreAudioBuffer + pos2, length2);
-		ringbuffer_processing_consume_bytes (devpCoreAudioRingBuffer, length2);
+		plrDriverAPI->ringbufferAPI->processing_consume_bytes (devpCoreAudioRingBuffer, length2);
 		len -= length2;
 		stream += length2;
 		lastLength += length2 >> 2 /* stereo + bit16 */;
@@ -221,7 +224,7 @@ static OSStatus theRenderProc(void *inRefCon, AudioUnitRenderActionFlags *inActi
 
 	if (len)
 	{
-		bzero (stream, len);
+		memset (stream, 0, len);
 		debug_printf ("theRenderProc: buffer overrun - %d left\n", len);
 	}
 
@@ -237,7 +240,7 @@ static void devpCoreAudioGetBuffer (void **buf, unsigned int *samples)
 
 	pthread_mutex_lock(&mutex);
 
-	ringbuffer_get_head_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
+	plrDriverAPI->ringbufferAPI->get_head_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
 
 	debug_printf ("devpCoreAudioGetBuffer: %d + %d\n", length1, length2);
 
@@ -266,7 +269,7 @@ static unsigned int devpCoreAudioIdle(void)
 		signed int expect_left;
 		signed int consume;
 
-		ringbuffer_get_tail_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
+		plrDriverAPI->ringbufferAPI->get_tail_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
 
 		clock_gettime (CLOCK_MONOTONIC, &tp);
 		curTime = tp.tv_sec * 1000000;
@@ -281,25 +284,25 @@ static unsigned int devpCoreAudioIdle(void)
 		consume = (signed int)(length1 + length2) - expect_left;
 		if (consume > 0)
 		{
-			ringbuffer_tail_consume_samples (devpCoreAudioRingBuffer, consume);
+			plrDriverAPI->ringbufferAPI->tail_consume_samples (devpCoreAudioRingBuffer, consume);
 			debug_printf ("devpCoreAudioIdle: time %" PRIu32" - %" PRIu32" => preconsume %d\n", lastCallbackTime, curTime, consume);
 		}
 	}
 /* STOP */
 
-	ringbuffer_get_tailandprocessing_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
+	plrDriverAPI->ringbufferAPI->get_tailandprocessing_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
 
 	/* do we need to insert pause-samples? */
 	if (devpCoreAudioInPause)
 	{
 		int pos1, length1, pos2, length2;
-		ringbuffer_get_head_bytes (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
-		bzero ((char *)devpCoreAudioBuffer+pos1, length1);
+		plrDriverAPI->ringbufferAPI->get_head_bytes (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
+		memset ((char *)devpCoreAudioBuffer+pos1, 0, length1);
 		if (length2)
 		{
-			bzero ((char *)devpCoreAudioBuffer+pos2, length2);
+			memset ((char *)devpCoreAudioBuffer+pos2, 0, length2);
 		}
-		ringbuffer_head_add_bytes (devpCoreAudioRingBuffer, length1 + length2);
+		plrDriverAPI->ringbufferAPI->head_add_pause_bytes (devpCoreAudioRingBuffer, length1 + length2);
 		devpCoreAudioPauseSamples += (length1 + length2) >> 2; /* stereo + 16bit */
 	}
 
@@ -333,14 +336,13 @@ static void devpCoreAudioOnBufferCallback (int samplesuntil, void (*callback)(vo
 
 	pthread_mutex_lock(&mutex);
 
-	ringbuffer_add_tail_callback_samples (devpCoreAudioRingBuffer, samplesuntil, callback, arg);
+	plrDriverAPI->ringbufferAPI->add_tail_callback_samples (devpCoreAudioRingBuffer, samplesuntil, callback, arg);
 
 	pthread_mutex_unlock(&mutex);
 
 	debug_printf ("devpCoreAudioOnBufferCallback: EXIT\n");
 }
 
-
 static void devpCoreAudioCommitBuffer (unsigned int samples)
 {
 	debug_printf ("devpCoreAudioCommitBuffer: ENTER\n");
@@ -349,7 +351,7 @@ static void devpCoreAudioCommitBuffer (unsigned int samples)
 
 	debug_printf ("devpCoreAudioCommitBuffer: %u\n", samples);
 
-	ringbuffer_head_add_samples (devpCoreAudioRingBuffer, samples);
+	plrDriverAPI->ringbufferAPI->head_add_samples (devpCoreAudioRingBuffer, samples);
 
 	pthread_mutex_unlock(&mutex);
 
@@ -362,7 +364,7 @@ static void devpCoreAudioPause (int pause)
 	devpCoreAudioInPause = pause;
 }
 
-static void devpCoreAudioStop(void)
+static void devpCoreAudioStop (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	OSErr status;
 
@@ -376,8 +378,9 @@ static void devpCoreAudioStop(void)
 
 	free (devpCoreAudioBuffer);
 	devpCoreAudioBuffer = 0;
-	ringbuffer_free (devpCoreAudioRingBuffer);
+	plrDriverAPI->ringbufferAPI->free (devpCoreAudioRingBuffer);
 	devpCoreAudioRingBuffer = 0;
+	cpifaceSession->plrActive = 0;
 }
 
 static void devpCoreAudioPeekBuffer (void **buf1, unsigned int *buf1length, void **buf2, unsigned int *buf2length)
@@ -389,7 +392,7 @@ static void devpCoreAudioPeekBuffer (void **buf1, unsigned int *buf1length, void
 	debug_printf ("devpCoreAudioPeekBuffer: ENTER\n");
 
 	pthread_mutex_lock(&mutex);
-	ringbuffer_get_tailandprocessing_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
+	plrDriverAPI->ringbufferAPI->get_tailandprocessing_samples (devpCoreAudioRingBuffer, &pos1, &length1, &pos2, &length2);
 	pthread_mutex_unlock(&mutex);
 
 	debug_printf ("devpCoreAudioPeekBuffer: EXIT\n");
@@ -433,7 +436,7 @@ static int devpCoreAudioPlay (uint32_t *rate, enum plrRequestFormat *format, str
 	*rate = devpCoreAudioRate; /* fixed */
 	*format = PLR_STEREO_16BIT_SIGNED; /* fixed */
 
-	plrbufsize = cfGetProfileInt2(cfSoundSec, "sound", "plrbufsize", 200, 10);
+	plrbufsize = cpifaceSession->configAPI->GetProfileInt2 (cpifaceSession->configAPI->SoundSec, "sound", "plrbufsize", 200, 10);
 	/* clamp the plrbufsize to be atleast 150ms and below 1000 ms */
 	if (plrbufsize < 150)
 	{
@@ -450,7 +453,7 @@ static int devpCoreAudioPlay (uint32_t *rate, enum plrRequestFormat *format, str
 		return 0;
 	}
 
-	if (!(devpCoreAudioRingBuffer = ringbuffer_new_samples (RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_SIGNED | RINGBUFFER_FLAGS_PROCESS, buflength)))
+	if (!(devpCoreAudioRingBuffer = plrDriverAPI->ringbufferAPI->new_samples (RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_SIGNED | RINGBUFFER_FLAGS_PROCESS, buflength)))
 	{
 		free (devpCoreAudioBuffer); devpCoreAudioBuffer = 0;
 		return 0;
@@ -462,16 +465,26 @@ static int devpCoreAudioPlay (uint32_t *rate, enum plrRequestFormat *format, str
 	{
 		fprintf(stderr, "[CoreAudio] AudioOutputUnitStart returned %d (%s)\n", (int)status, OSStatus_to_string(status));
 		free (devpCoreAudioBuffer); devpCoreAudioBuffer = 0;
-		ringbuffer_free (devpCoreAudioRingBuffer); devpCoreAudioRingBuffer = 0;
+		plrDriverAPI->ringbufferAPI->free (devpCoreAudioRingBuffer); devpCoreAudioRingBuffer = 0;
 		return 0;
 	}
 
-	cpifaceSession->GetMasterSample = plrGetMasterSample;
-	cpifaceSession->GetRealMasterVolume = plrGetRealMasterVolume;
+	cpifaceSession->GetMasterSample = plrDriverAPI->GetMasterSample;
+	cpifaceSession->GetRealMasterVolume = plrDriverAPI->GetRealMasterVolume;
+	cpifaceSession->plrActive = 1;
 
 	return 1;
 }
 
+static void devpCoreAudioGetStats (uint64_t *committed, uint64_t *processed)
+{
+	pthread_mutex_lock(&mutex);
+
+	plrDriverAPI->ringbufferAPI->get_stats (devpCoreAudioRingBuffer, committed, processed);
+
+	pthread_mutex_unlock(&mutex);
+}
+
 static const struct plrDevAPI_t devpCoreAudio = {
 	devpCoreAudioIdle,
 	devpCoreAudioPeekBuffer,
@@ -482,24 +495,23 @@ static const struct plrDevAPI_t devpCoreAudio = {
 	devpCoreAudioCommitBuffer,
 	devpCoreAudioPause,
 	devpCoreAudioStop,
-	0
+	0, /* VolRegs */
+	0, /* ProcessKey */
+	devpCoreAudioGetStats
 };
 
-static int CoreAudioInit(const struct deviceinfo *c)
+static const struct plrDevAPI_t *CoreAudioInit (const struct plrDriver_t *driver, const struct plrDriverAPI_t *DriverAPI)
 {
-	plrDevAPI = &devpCoreAudio;
-	return 1;
+	plrDriverAPI = DriverAPI;
+
+	return &devpCoreAudio;
 }
 
-static void CoreAudioClose(void)
+static void CoreAudioClose (const struct plrDriver_t *driver)
 {
-	if (plrDevAPI == &devpCoreAudio)
-	{
-		plrDevAPI = 0;
-	}
 }
 
-static int CoreAudioDetect(struct deviceinfo *card)
+static int CoreAudioDetect (const struct plrDriver_t *driver)
 {
 	AudioStreamBasicDescription inDesc;
 	const int channels=2;
@@ -659,12 +671,6 @@ static int CoreAudioDetect(struct deviceinfo *card)
 	}
 
 	/* ao is now created, the above is needed only ONCE */
-	card->devtype=&plrCoreAudio;
-	card->port=0;
-	card->port2=0;
-	card->subtype=-1;
-	card->mem=0;
-	card->chan=2;
 
 	needfini=1;
 
@@ -689,7 +695,25 @@ static void __attribute__((destructor))fini(void)
 	}
 }
 
-struct sounddevice plrCoreAudio={SS_PLAYER, 1, "CoreAudio player", CoreAudioDetect,  CoreAudioInit,  CoreAudioClose};
+static int CoreAudioPluginInit (struct PluginInitAPI_t *API)
+{
+	API->plrRegisterDriver (&plrCoreAudio);
 
-const char *dllinfo="driver plrCoreAudio";
-DLLEXTINFO_DRIVER_PREFIX struct linkinfostruct dllextinfo = {.name = "devpcoreaudio", .desc = "OpenCP Player Device: CoreAudio (c) 2006-'22 Stian Skjelstad", .ver = DLLVERSION, .sortindex = 99};
+	return errOk;
+}
+
+static void CoreAudioWriterPluginClose (struct PluginCloseAPI_t *API)
+{
+	API->plrUnregisterDriver (&plrCoreAudio);
+}
+
+static const struct plrDriver_t  plrCoreAudio =
+{
+	"devpCoreaudio",
+	"CoreAudio player",
+	CoreAudioDetect,
+	CoreAudioInit,
+	CoreAudioClose
+};
+
+DLLEXTINFO_DRIVER_PREFIX struct linkinfostruct dllextinfo = {.name = "devpcoreaudio", .desc = "OpenCP Player Device: CoreAudio (c) 2006-'24 Stian Skjelstad", .ver = DLLVERSION, .sortindex = 99, .PluginInit = CoreAudioPluginInit, .PluginClose = CoreAudioWriterPluginClose};
