@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Sebastian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Sebastian Skjelstad <stian.skjelstad@gmail.com>
  *
  * UNIX cdrom filebrowser
  *
@@ -121,7 +121,7 @@ static void try(const char *dev, const char *vdev)
 #ifdef CDROM_VERBOSE
 	fprintf(stderr, "Testing %s\n", dev);
 #endif
-	bzero (temppath, sizeof (temppath));
+	memset (temppath, 0, sizeof (temppath));
 	if (readlink (dev, temppath, sizeof (temppath) - 1) > 0)
 	{
 		if (strncmp (temppath, "/dev/sr", 7)) return;
@@ -257,11 +257,6 @@ static void *cdrom_thread (void *_self)
 	pthread_mutex_lock (&self->mutex);
 	while (1)
 	{
-		if (self->shutdown)
-		{
-			pthread_mutex_unlock (&self->mutex);
-			return 0;
-		}
 		if (self->request)
 		{
 			struct cdrom_read_audio rip_ioctl;
@@ -278,11 +273,16 @@ static void *cdrom_thread (void *_self)
 			pthread_mutex_lock (&self->mutex);
 			self->request_complete = 1;
 		}
+		if (self->shutdown) /* check this before entering cond_wait */
+		{
+			pthread_mutex_unlock (&self->mutex);
+			return 0;
+		}
 		pthread_cond_wait (&self->cond, &self->mutex);
 	}
 }
 
-static int cdint(void)
+static int cdint (const struct configAPI_t *configAPI)
 {
 	char dev[32], vdev[12];
 	char a;
@@ -304,7 +304,8 @@ static int cdint(void)
 		dirdbFindAndRef (DIRDB_NOPARENT, "cdrom:", dirdb_use_dir),
 		0, /* we ignore refcounting */
 		0, /* not an archive */
-		0  /* not a playlist */);
+		0, /* not a playlist */
+		COMPRESSION_NONE);
 
 	dmCDROM=RegisterDrive("cdrom:", &cdrom_root, &cdrom_root);
 
@@ -375,8 +376,8 @@ static void cdclose(void)
 
 		pthread_mutex_lock (&cdroms[i].mutex);
 		cdroms[i].shutdown = 1;
-		pthread_mutex_unlock (&cdroms[i].mutex);
 		pthread_cond_signal (&cdroms[i].cond);
+		pthread_mutex_unlock (&cdroms[i].mutex);
 		pthread_join (cdroms[i].thread, &retval);
 
 		close (cdroms[i].fd);
@@ -464,7 +465,8 @@ static int cdrom_root_readdir_iterate (ocpdirhandle_pt _dh)
 		dirdbFindAndRef (dh->owner->dirdb_ref, cdroms[dh->n].vdev, dirdb_use_dir),
 		1,
 		0, /* not an archive */
-		0  /* not a playlist */);
+		0, /* not a playlist */
+		COMPRESSION_NONE);
 
 	dir->cdrom = cdroms + dh->n;
 	dh->owner->ref (dh->owner);
@@ -529,7 +531,8 @@ static struct ocpdir_t *cdrom_root_readdir_dir (struct ocpdir_t *_self, uint32_t
 			dirdbRef (dirdb_ref, dirdb_use_dir),
 			1,
 			0, /* not an archive */
-			0  /* not a playlist */);
+			0, /* not a playlist */
+		        COMPRESSION_NONE);
 
 		return &dir->head;
 	}
@@ -589,7 +592,7 @@ static ocpdirhandle_pt cdrom_drive_readdir_start (struct ocpdir_t *_self, void(*
 	dh->owner = self;
 	dh->initlba = -1;
 
-	bzero (&self->cdrom->lasttoc, sizeof (self->cdrom->lasttoc));
+	memset (&self->cdrom->lasttoc, 0, sizeof (self->cdrom->lasttoc));
 	free (dh->owner->cdrom->lastdiscid);
 	free (dh->owner->cdrom->lasttocstr);
 	dh->owner->cdrom->lastdiscid = 0;
@@ -684,7 +687,7 @@ leadout:
 			goto failout;
 		}
 
-		bzero (offsets, sizeof (offsets));
+		memset (offsets, 0, sizeof (offsets));
 		for (i=first; i <= last; i++)
 		{
 			offsets[i] = dh->owner->cdrom->lasttoc.track[i].lba_addr + 150;
@@ -790,7 +793,8 @@ static int cdrom_drive_readdir_iterate (ocpdirhandle_pt _dh)
 			                 cdrom_track_filename_override_disc,
 			                 dirdbFindAndRef (dh->owner->head.dirdb_ref, filename, dirdb_use_file),
 			                 1, /* refcount */
-			                 1  /* is_nodetect */);
+			                 1  /* is_nodetect */,
+			                 COMPRESSION_NONE);
 
 			dh->owner->head.ref (&dh->owner->head);
 			file->cdrom = dh->owner->cdrom;
@@ -864,7 +868,8 @@ static int cdrom_drive_readdir_iterate (ocpdirhandle_pt _dh)
 		                 cdrom_track_filename_override_track,
 		                 dirdbFindAndRef (dh->owner->head.dirdb_ref, filename, dirdb_use_file),
 		                 1, /* refcount */
-		                 1  /* is_nodetect */);
+		                 1, /* is_nodetect */
+		                 COMPRESSION_NONE);
 
 		dh->owner->head.ref (&dh->owner->head);
 		file->cdrom = dh->owner->cdrom;
@@ -886,7 +891,7 @@ static int cdrom_drive_readdir_iterate (ocpdirhandle_pt _dh)
 				{
 					snprintf (mi.album, sizeof (mi.album), "%s", dh->owner->cdrom->musicbrainzdata->album);
 				}
-				if (dh->owner->cdrom->musicbrainzdata && dh->owner->cdrom->musicbrainzdata->title[dh->i])
+				if (dh->owner->cdrom->musicbrainzdata && dh->owner->cdrom->musicbrainzdata->title[dh->i][0])
 				{
 					snprintf (mi.title, sizeof (mi.title), "%s", dh->owner->cdrom->musicbrainzdata->title[dh->i]);
 				} else if (!mi.title[0])
@@ -941,20 +946,18 @@ struct ocpfilehandle_cdrom_track_t
 {
 	struct ocpfilehandle_t head;
 	struct cdrom_track_ocpfile_t *owner;
-	int refcount;
 };
 
 static void ocpfilehandle_cdrom_track_ref (struct ocpfilehandle_t *_handle)
 {
-	struct ocpfilehandle_cdrom_track_t *handle = (struct ocpfilehandle_cdrom_track_t *)_handle;
-	handle->refcount++;
+	_handle->refcount++;
 }
 
 static void ocpfilehandle_cdrom_track_unref (struct ocpfilehandle_t *_handle)
 {
 	struct ocpfilehandle_cdrom_track_t *handle = (struct ocpfilehandle_cdrom_track_t *)_handle;
-	handle->refcount--;
-	if (!handle->refcount)
+	handle->head.refcount--;
+	if (!handle->head.refcount)
 	{
 		handle->owner->head.unref (&handle->owner->head);
 		dirdbUnref (handle->head.dirdb_ref, dirdb_use_filehandle);
@@ -1011,8 +1014,8 @@ static int ocpfilehandle_cdrom_track_ioctl (struct ocpfilehandle_t *_handle, con
 		self->request_returnvalue = 0;
 		self->request_complete = 0;
 
-		pthread_mutex_unlock (&self->mutex);
 		pthread_cond_signal (&self->cond);
+		pthread_mutex_unlock (&self->mutex);
 		return 1; /* not ready */
 	}
 	if (!strcmp (cmd, IOCTL_CDROM_READAUDIO_ASYNC_PULL))
@@ -1074,8 +1077,6 @@ static struct ocpfilehandle_t *cdrom_track_open (struct ocpfile_t *_self)
 	                      ocpfilehandle_cdrom_track_unref,
 	                      &self->head,
 	                      ocpfilehandle_cdrom_track_seek,
-	                      ocpfilehandle_cdrom_track_seek,
-	                      ocpfilehandle_cdrom_track_seek,
 	                      ocpfilehandle_cdrom_track_getpos,
 	                      ocpfilehandle_cdrom_track_eof,
 	                      ocpfilehandle_cdrom_track_error,
@@ -1084,14 +1085,13 @@ static struct ocpfilehandle_t *cdrom_track_open (struct ocpfile_t *_self)
 	                      ocpfilehandle_cdrom_track_filesize,
 	                      ocpfilehandle_cdrom_track_filesize_ready,
 	                      ocpfilehandle_cdrom_track_filename_override_disc,
-	                      _self->dirdb_ref);
+	                      _self->dirdb_ref,
+	                      1 /* refcount */);
 	dirdbRef (_self->dirdb_ref, dirdb_use_filehandle);
 
 	retval->owner = self;
 	retval->owner->head.ref (&retval->owner->head);
 
-	retval->refcount = 1;
-
 	return &retval->head;
 }
 
@@ -1111,4 +1111,4 @@ static int cdrom_track_filesize_ready (struct ocpfile_t *_self)
 	return 1;
 }
 
-DLLEXTINFO_CORE_PREFIX struct linkinfostruct dllextinfo = {.name = "cdrom", .desc = "OpenCP UNIX audio-cdrom filebrowser (c) 2004-'22 Stian Skjelstad", .ver = DLLVERSION, .Init = cdint, .Close = cdclose, .sortindex = 30};
+DLLEXTINFO_CORE_PREFIX struct linkinfostruct dllextinfo = {.name = "cdrom", .desc = "OpenCP UNIX audio-cdrom filebrowser (c) 2004-'24 Stian Skjelstad", .ver = DLLVERSION, .Init = cdint, .Close = cdclose, .sortindex = 30};
