@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * database for storing a tree of filenames
  *
@@ -325,7 +325,7 @@ int dirdbInit (const struct configAPI_t *configAPI)
 	dumpdirdb();
 #endif
 
-	osfile_purge_readaheadcache (dirdbFile);
+	osfile_purge_readahead_cache (dirdbFile);
 
 	fprintf(stderr, "Done\n");
 	return 1;
@@ -348,7 +348,7 @@ unload:
 		dirdbData[i].next = dirdbFreeChild;
 		dirdbFreeChild = i;
 	}
-	osfile_purge_readaheadcache (dirdbFile);
+	osfile_purge_readahead_cache (dirdbFile);
 	return retval;
 }
 
