@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Code to decompress BZIP2 files using libbz2
  *
@@ -96,6 +96,8 @@ static int bzip2_ocpfilehandle_compressInit (struct bzip2_ocpfilehandle_t *s)
 {
 	int retval;
 
+	DEBUG_PRINT ("bzip2_ocpfilehandle_compressInit (s->need_deinit=%d)\n", s->need_deinit);
+
 	if (s->need_deinit)
 	{
 		BZ2_bzDecompressEnd (&s->strm);
@@ -155,6 +157,8 @@ static void bzip2_ocpfilehandle_unref (struct ocpfilehandle_t *_s)
 		return;
 	}
 
+	DEBUG_PRINT("bzip2_ocpfilehandle_unref, last instance, s->need_deinit=%d\n", s->need_deinit);
+
 	if (s->need_deinit)
 	{
 		BZ2_bzDecompressEnd (&s->strm);
@@ -183,6 +187,8 @@ static int bzip2_ocpfilehandle_seek_set (struct ocpfilehandle_t *_s, int64_t pos
 
 	if (pos < 0) return -1;
 
+	DEBUG_PRINT ("bzip2_ocpfilehandle_seek_set pos=%"PRId64" oldpos=%"PRId64" realpos=%"PRId64" s->owner->filesize_pending=%d \n", pos, s->pos, s->realpos, s->owner->filesize_pending);
+
 	if (s->owner->filesize_pending)
 	{
 		if (pos > s->pos)
@@ -203,68 +209,12 @@ static int bzip2_ocpfilehandle_seek_set (struct ocpfilehandle_t *_s, int64_t pos
 	return 0;
 }
 
-static int bzip2_ocpfilehandle_seek_cur (struct ocpfilehandle_t *_s, int64_t pos)
-{
-	struct bzip2_ocpfilehandle_t *s = (struct bzip2_ocpfilehandle_t *)_s;
-
-	if (pos <= 0)
-	{
-		if (pos == INT64_MIN) return -1; /* we never have files this size */
-		if ((-pos) > s->pos) return -1;
-		s->pos += pos;
-	} else {
-		/* check for overflow */
-		if ((int64_t)(pos + s->pos) < 0) return -1;
-
-		if (s->owner->filesize_pending)
-		{
-			if (_s->filesize (_s) == FILESIZE_ERROR) /* force the size to be calculated */
-			{
-				s->error = 1;
-				return -1;
-			}
-		}
-
-		if ((pos + s->pos) > s->owner->uncompressed_filesize) return -1;
-		s->pos += pos;
-	}
-
-	s->error = 0;
-
-	return 0;
-}
-
-static int bzip2_ocpfilehandle_seek_end (struct ocpfilehandle_t *_s, int64_t pos)
-{
-	struct bzip2_ocpfilehandle_t *s = (struct bzip2_ocpfilehandle_t *)_s;
-
-	if (pos > 0) return -1;
-
-	if (pos == INT64_MIN) return -1; /* we never have files this size */
-
-	if (s->owner->filesize_pending)
-	{
-		if (_s->filesize (_s) == FILESIZE_ERROR) /* force the size to be calculated */
-		{
-			s->error = 1;
-			return -1;
-		}
-	}
-
-	if (pos < -(int64_t)(s->owner->uncompressed_filesize)) return -1;
-
-	s->pos = s->owner->uncompressed_filesize + pos;
-
-	s->error = 0;
-
-	return 0;
-}
-
-
 static uint64_t bzip2_ocpfilehandle_getpos (struct ocpfilehandle_t *_s)
 {
 	struct bzip2_ocpfilehandle_t *s = (struct bzip2_ocpfilehandle_t *)_s;
 
+	DEBUG_PRINT ("bzip2_ocpfilehandle_getpos pos=%"PRId64"\n", s->pos);
+
 	return s->pos;
 }
 
@@ -272,6 +222,8 @@ static int bzip2_ocpfilehandle_eof (struct ocpfilehandle_t *_s)
 {
 	struct bzip2_ocpfilehandle_t *s = (struct bzip2_ocpfilehandle_t *)_s;
 
+	DEBUG_PRINT ("bzip2_ocpfilehandle_eof\n");
+
 	if (!s->owner->filesize_pending)
 	{
 		if (_s->filesize (_s) == FILESIZE_ERROR) /* force the size to be calculated */
@@ -287,6 +239,8 @@ static int bzip2_ocpfilehandle_error (struct ocpfilehandle_t *_s)
 {
 	struct bzip2_ocpfilehandle_t *s = (struct bzip2_ocpfilehandle_t *)_s;
 
+	DEBUG_PRINT ("bzip2_ocpfilehandle_error\n");
+
 	return s->error;
 }
 
@@ -296,6 +250,8 @@ static int bzip2_ocpfilehandle_read (struct ocpfilehandle_t *_s, void *dst, int
 	int retval = 0;
 	int recall = 0;
 
+	DEBUG_PRINT ("bzip2_ocpfilehandle_read len=%d pos=%"PRId64" realpos=%"PRId64"\n", len, s->pos, s->realpos);
+
 	/* do we need to reverse? */
 	if ((s->pos < s->realpos) || (!s->need_deinit))
 	{
@@ -469,7 +425,7 @@ static int bzip2_ocpfilehandle_read (struct ocpfilehandle_t *_s, void *dst, int
 
 				dirdbGetName_internalstr (s->compressedfilehandle->dirdb_ref, &filename);
 
-				DEBUG_PRINT ("[BZIP2 filehandle_read EOF] adbMetaAdd(%s, %"PRId64", BZIP2, [%02x %02x %02x %02x %02x %02x %02x %02x] (%"PRIu64" + %d => %"PRIu64")\n", filename, compressedfile_size, buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5], buffer[6], buffer[7], s->realpos, s->outputbuffer_fill, filesize);
+				DEBUG_PRINT ("[BZIP2 filehandle_read EOF] adbMetaAdd(%s, %"PRIu64", BZIP2, [%02x %02x %02x %02x %02x %02x %02x %02x] (%"PRIu64" + %d => %"PRIu64")\n", filename, compressedfile_size, buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5], buffer[6], buffer[7], s->realpos, s->outputbuffer_fill, filesize);
 				adbMetaAdd (filename, compressedfile_size, "BZIP2", buffer, 8);
 			}
 
@@ -521,8 +477,6 @@ static struct ocpfilehandle_t *bzip2_ocpfile_open (struct ocpfile_t *_s)
 	                       bzip2_ocpfilehandle_unref,
 	                      &s->head,
 	                       bzip2_ocpfilehandle_seek_set,
-	                       bzip2_ocpfilehandle_seek_cur,
-	                       bzip2_ocpfilehandle_seek_end,
 	                       bzip2_ocpfilehandle_getpos,
 	                       bzip2_ocpfilehandle_eof,
 	                       bzip2_ocpfilehandle_error,
@@ -531,7 +485,8 @@ static struct ocpfilehandle_t *bzip2_ocpfile_open (struct ocpfile_t *_s)
 	                       bzip2_ocpfilehandle_filesize,
 	                       bzip2_ocpfilehandle_filesize_ready,
 	                       0, /* filename_override */
-	                       dirdbRef (s->head.dirdb_ref, dirdb_use_filehandle));
+	                       dirdbRef (s->head.dirdb_ref, dirdb_use_filehandle),
+	                       1 /* refcount */);
 
 	retval->owner = s;
 	s->head.ref (&s->head);
@@ -545,8 +500,6 @@ static struct ocpfilehandle_t *bzip2_ocpfile_open (struct ocpfile_t *_s)
 		return 0;
 	}
 
-	retval->head.refcount = 1;
-
 	return &retval->head;
 }
 
@@ -581,7 +534,7 @@ static uint64_t bzip2_ocpfile_filesize (struct ocpfile_t *_s)
 		compressedfile_size = s->compressedfile->filesize (s->compressedfile);
 
 		uint8_t *metadata = 0;
-		size_t metadatasize = 0;
+		uint32_t metadatasize = 0;
 
 		if ((compressedfile_size < 12) || (compressedfile_size == FILESIZE_ERROR) || (compressedfile_size == FILESIZE_STREAM))
 		{
@@ -605,9 +558,11 @@ static uint64_t bzip2_ocpfile_filesize (struct ocpfile_t *_s)
 				                           ((uint64_t)(metadata[0]));
 				free (metadata);
 
-				DEBUG_PRINT ("[BZIP2 ocpfile_filesize]: got metadatasize=0x%08lx %02x %02x %02x %02x %02x %02x %02x %02x => %"PRIu64"\n", metadatasize, metadata[0], metadata[1], metadata[2], metadata[3], metadata[4], metadata[5], metadata[6], metadata[7], s->uncompressed_filesize);
+				DEBUG_PRINT ("[BZIP2 ocpfile_filesize]: got metadatasize=0x%08"PRIu32" %02x %02x %02x %02x %02x %02x %02x %02x => %"PRIu64"\n", metadatasize, metadata[0], metadata[1], metadata[2], metadata[3], metadata[4], metadata[5], metadata[6], metadata[7], s->uncompressed_filesize);
 
 				return s->uncompressed_filesize;
+			} else {
+				DEBUG_PRINT ("[BZIP2 ocpfile_filesize]: got metadatasize=0x%08"PRIu32", unexpected size\n", metadatasize);
 			}
 			free (metadata); /* wrong size???... */
 			metadata = 0;
@@ -841,7 +796,8 @@ static struct ocpdir_t *bzip2_check_steal (struct ocpfile_t *s, const uint32_t d
 	                s->dirdb_ref,
 	                1, /* refcount */
 	                1, /* is_archive */
-	                0  /* is_playlist */);
+	                0, /* is_playlist */
+	                s->compression);
 
 	s->parent->ref (s->parent);
 	dirdbRef (s->dirdb_ref, dirdb_use_dir);
@@ -856,7 +812,8 @@ static struct ocpdir_t *bzip2_check_steal (struct ocpfile_t *s, const uint32_t d
 	                 0, /* filename_override */
 	                 dirdb_ref,
 	                 0, /* refcount */
-	                 0  /* is_nodetect */);
+	                 0, /* is_nodetect */
+	                 COMPRESSION_ADD_STREAM(s->compression));
 
 	retval->child.filesize_pending = 1;
 	retval->child.uncompressed_filesize = 0;
@@ -867,7 +824,7 @@ static struct ocpdir_t *bzip2_check_steal (struct ocpfile_t *s, const uint32_t d
 	if (s->filesize_ready (s))
 	{
 		unsigned char *metadata = 0;
-		size_t metadatasize = 0;
+		uint32_t metadatasize = 0;
 		const char *filename = 0;
 
 		dirdbGetName_internalstr (retval->child.compressedfile->dirdb_ref, &filename);
@@ -886,8 +843,9 @@ static struct ocpdir_t *bzip2_check_steal (struct ocpfile_t *s, const uint32_t d
 				                                      ((uint64_t)(metadata[1]) << 8) |
 				                                      ((uint64_t)(metadata[0]));
 
-				DEBUG_PRINT ("[BZIP2 bzip2_check_steal]: got metadatasize=0x%08lx %02x %02x %02x %02x %02x %02x %02x %02x => %"PRIu64"\n", metadatasize, metadata[0], metadata[1], metadata[2], metadata[3], metadata[4], metadata[5], metadata[6], metadata[7], retval->child.uncompressed_filesize);
-
+				DEBUG_PRINT ("[BZIP2 bzip2_check_steal]: got metadatasize=0x%08"PRIu32" %02x %02x %02x %02x %02x %02x %02x %02x => %"PRIu64"\n", metadatasize, metadata[0], metadata[1], metadata[2], metadata[3], metadata[4], metadata[5], metadata[6], metadata[7], retval->child.uncompressed_filesize);
+			} else {
+				DEBUG_PRINT ("[BZIP2 bzip2_check_steal]: got metadatasize=0x%08"PRIu32", an unexpected size\n", metadatasize);
 			}
 			free (metadata);
 			metadata = 0;
