@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Code to hold a virtual static directory
  *
@@ -181,7 +181,8 @@ struct ocpdir_mem_t *ocpdir_mem_alloc (struct ocpdir_t *parent, const char *name
 		dirdbFindAndRef (parent?parent->dirdb_ref:DIRDB_NOPARENT, name, dirdb_use_dir),
 		1, /* refcount */
 		0, /* is_archive */
-		0  /* is_playlist */);
+		0, /* is_playlist */
+	        COMPRESSION_NONE);
 
 	if (parent)
 	{
