@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Code to decompress GZIP files using zLib
  *
@@ -205,64 +205,6 @@ static int gzip_ocpfilehandle_seek_set (struct ocpfilehandle_t *_s, int64_t pos)
 	return 0;
 }
 
-static int gzip_ocpfilehandle_seek_cur (struct ocpfilehandle_t *_s, int64_t pos)
-{
-	struct gzip_ocpfilehandle_t *s = (struct gzip_ocpfilehandle_t *)_s;
-
-	if (pos <= 0)
-	{
-		if (pos == INT64_MIN) return -1; /* we never have files this size */
-		if ((-pos) > s->pos) return -1;
-		s->pos += pos;
-	} else {
-		/* check for overflow */
-		if ((int64_t)(pos + s->pos) < 0) return -1;
-
-		if (s->owner->filesize_pending)
-		{
-			if (_s->filesize (_s) == FILESIZE_ERROR) /* force the size to be calculated */
-			{
-				s->error = 1;
-				return -1;
-			}
-		}
-
-		if ((pos + s->pos) > s->owner->uncompressed_filesize) return -1;
-		s->pos += pos;
-	}
-
-	s->error = 0;
-
-	return 0;
-}
-
-static int gzip_ocpfilehandle_seek_end (struct ocpfilehandle_t *_s, int64_t pos)
-{
-	struct gzip_ocpfilehandle_t *s = (struct gzip_ocpfilehandle_t *)_s;
-
-	if (pos > 0) return -1;
-
-	if (pos == INT64_MIN) return -1; /* we never have files this size */
-
-	if (s->owner->filesize_pending)
-	{
-		if (_s->filesize (_s) == FILESIZE_ERROR) /* force the size to be calculated */
-		{
-			s->error = 1;
-			return -1;
-		}
-	}
-
-	if (pos < -(int64_t)(s->owner->uncompressed_filesize)) return -1;
-
-	s->pos = s->owner->uncompressed_filesize + pos;
-
-	s->error = 0;
-
-	return 0;
-}
-
-
 static uint64_t gzip_ocpfilehandle_getpos (struct ocpfilehandle_t *_s)
 {
 	struct gzip_ocpfilehandle_t *s = (struct gzip_ocpfilehandle_t *)_s;
@@ -469,7 +411,7 @@ static int gzip_ocpfilehandle_read (struct ocpfilehandle_t *_s, void *dst, int l
 
 				dirdbGetName_internalstr (s->compressedfilehandle->dirdb_ref, &filename);
 
-				DEBUG_PRINT ("[GZIP filehandle_read EOF] adbMetaAdd(%s, %"PRId64", GZIP, [%02x %02x %02x %02x %02x %02x %02x %02x] (%"PRIu64" + %d => %"PRIu64")\n", filename, compressedfile_size, buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5], buffer[6], buffer[7], s->realpos, s->outputbuffer_fill, filesize);
+				DEBUG_PRINT ("[GZIP filehandle_read EOF] adbMetaAdd(%s, %"PRIu64", GZIP, [%02x %02x %02x %02x %02x %02x %02x %02x] (%"PRIu64" + %d => %"PRIu64")\n", filename, compressedfile_size, buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5], buffer[6], buffer[7], s->realpos, s->outputbuffer_fill, filesize);
 				adbMetaAdd (filename, compressedfile_size, "GZIP", buffer, 8);
 			}
 
@@ -521,8 +463,6 @@ static struct ocpfilehandle_t *gzip_ocpfile_open (struct ocpfile_t *_s)
 	                       gzip_ocpfilehandle_unref,
 	                      &s->head,
 	                       gzip_ocpfilehandle_seek_set,
-	                       gzip_ocpfilehandle_seek_cur,
-	                       gzip_ocpfilehandle_seek_end,
 	                       gzip_ocpfilehandle_getpos,
 	                       gzip_ocpfilehandle_eof,
 	                       gzip_ocpfilehandle_error,
@@ -531,7 +471,9 @@ static struct ocpfilehandle_t *gzip_ocpfile_open (struct ocpfile_t *_s)
 	                       gzip_ocpfilehandle_filesize,
 	                       gzip_ocpfilehandle_filesize_ready,
 	                       0, /* filename_override */
-	                       dirdbRef (s->head.dirdb_ref, dirdb_use_filehandle));
+	                       dirdbRef (s->head.dirdb_ref, dirdb_use_filehandle),
+	                       1
+	);
 
 	retval->owner = s;
 	s->head.ref (&s->head);
@@ -545,8 +487,6 @@ static struct ocpfilehandle_t *gzip_ocpfile_open (struct ocpfile_t *_s)
 		return 0;
 	}
 
-	retval->head.refcount = 1;
-
 	return &retval->head;
 }
 
@@ -581,7 +521,7 @@ static uint64_t gzip_ocpfile_filesize (struct ocpfile_t *_s)
 		compressedfile_size = s->compressedfile->filesize (s->compressedfile);
 
 		uint8_t *metadata = 0;
-		size_t metadatasize = 0;
+		uint32_t metadatasize = 0;
 
 		if ((compressedfile_size < 12) || (compressedfile_size == FILESIZE_ERROR) || (compressedfile_size == FILESIZE_STREAM))
 		{
@@ -605,9 +545,11 @@ static uint64_t gzip_ocpfile_filesize (struct ocpfile_t *_s)
 				                           ((uint64_t)(metadata[0]));
 				free (metadata);
 
-				DEBUG_PRINT ("[GZIP ocpfile_filesize]: got metadatasize=0x%08lx %02x %02x %02x %02x %02x %02x %02x %02x => %"PRIu64"\n", metadatasize, metadata[0], metadata[1], metadata[2], metadata[3], metadata[4], metadata[5], metadata[6], metadata[7], s->uncompressed_filesize);
+				DEBUG_PRINT ("[GZIP ocpfile_filesize]: got metadatasize=0x%08"PRIu32" %02x %02x %02x %02x %02x %02x %02x %02x => %"PRIu64"\n", metadatasize, metadata[0], metadata[1], metadata[2], metadata[3], metadata[4], metadata[5], metadata[6], metadata[7], s->uncompressed_filesize);
 
 				return s->uncompressed_filesize;
+			} else {
+				DEBUG_PRINT ("[GZIP ocpfile_filesize]: got metadatasize=0x%08"PRIu32", unexpected size\n", metadatasize);
 			}
 			free (metadata); /* wrong size???... */
 			metadata = 0;
@@ -643,7 +585,7 @@ static uint64_t gzip_ocpfile_filesize (struct ocpfile_t *_s)
 		}
 
 		/* If so, the original size (for the last member....) is stored at the end of the file... as 32bit, hence the test for LARGEST_THEORETICALLY_32BIT_SIZE */
-		if (h->seek_end (h, -4) < 0)
+		if (h->seek_set (h, h->filesize(h) - 4) < 0)
 		{
 			h->unref (h); h = 0;
 			return FILESIZE_ERROR;
@@ -897,7 +839,8 @@ static struct ocpdir_t *gzip_check_steal (struct ocpfile_t *s, const uint32_t di
 	                s->dirdb_ref,
 	                1, /* refcount */
 	                1, /* is_archive */
-	                0  /* is_playlist */);
+	                0, /* is_playlist */
+	                s->compression);
 
 	s->parent->ref (s->parent);
 	dirdbRef (s->dirdb_ref, dirdb_use_dir);
@@ -912,7 +855,8 @@ static struct ocpdir_t *gzip_check_steal (struct ocpfile_t *s, const uint32_t di
 	                 0, /* filename_override */
 	                 dirdb_ref,
 	                 1, /* refcount */
-	                 0  /* is_nodetect */);
+	                 0, /* is_nodetect */
+	                 COMPRESSION_ADD_STREAM(s->compression));
 
 	retval->child.filesize_pending = 1;
 	retval->child.uncompressed_filesize = 0;
@@ -923,7 +867,7 @@ static struct ocpdir_t *gzip_check_steal (struct ocpfile_t *s, const uint32_t di
 	if (s->filesize_ready (s))
 	{
 		unsigned char *metadata = 0;
-		size_t metadatasize = 0;
+		uint32_t metadatasize = 0;
 		const char *filename = 0;
 
 		dirdbGetName_internalstr (retval->child.compressedfile->dirdb_ref, &filename);
@@ -942,8 +886,9 @@ static struct ocpdir_t *gzip_check_steal (struct ocpfile_t *s, const uint32_t di
 				                                      ((uint64_t)(metadata[1]) << 8) |
 				                                      ((uint64_t)(metadata[0]));
 
-				DEBUG_PRINT ("[GZIP gzip_check_steal]: got metadatasize=0x%08lx %02x %02x %02x %02x %02x %02x %02x %02x => %"PRIu64"\n", metadatasize, metadata[0], metadata[1], metadata[2], metadata[3], metadata[4], metadata[5], metadata[6], metadata[7], retval->child.uncompressed_filesize);
-
+				DEBUG_PRINT ("[GZIP gzip_check_steal]: got metadatasize=0x%08"PRIu32" %02x %02x %02x %02x %02x %02x %02x %02x => %"PRIu64"\n", metadatasize, metadata[0], metadata[1], metadata[2], metadata[3], metadata[4], metadata[5], metadata[6], metadata[7], retval->child.uncompressed_filesize);
+			} else {
+				DEBUG_PRINT ("[GZIP gzip_check_steal]: got metadatasize=0x%08"PRIu32", unexpected size\n", metadatasize);
 			}
 			free (metadata);
 			metadata = 0;
