@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2022-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2022-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Code to decode PAK (Westwood and Quake) file archives
  *
@@ -126,8 +126,6 @@ static int pak_file_filesize_ready (struct ocpfile_t *);
 static void pak_filehandle_ref (struct ocpfilehandle_t *);
 static void pak_filehandle_unref (struct ocpfilehandle_t *);
 static int pak_filehandle_seek_set (struct ocpfilehandle_t *, int64_t pos);
-static int pak_filehandle_seek_cur (struct ocpfilehandle_t *, int64_t pos);
-static int pak_filehandle_seek_end (struct ocpfilehandle_t *, int64_t pos);
 static uint64_t pak_filehandle_getpos (struct ocpfilehandle_t *);
 static int pak_filehandle_eof (struct ocpfilehandle_t *);
 static int pak_filehandle_error (struct ocpfilehandle_t *);
@@ -164,7 +162,7 @@ static uint32_t pak_instance_add (struct pak_instance_t *self,
                                   const uint32_t  fileoffset);
 
 /* in the blob, we will switch / into \0 temporary as we parse them */
-static void pak_instance_decode_blob (struct pak_instance_t *self, uint8_t *blob, size_t blobsize)
+static void pak_instance_decode_blob (struct pak_instance_t *self, uint8_t *blob, uint32_t blobsize)
 {
 
 	uint8_t *eos;
@@ -220,7 +218,7 @@ static void pak_instance_decode_blob (struct pak_instance_t *self, uint8_t *blob
 #endif
 }
 
-static void pak_instance_encode_blob (struct pak_instance_t *self, uint8_t **blob, size_t *blobfill)
+static void pak_instance_encode_blob (struct pak_instance_t *self, uint8_t **blob, uint32_t *blobfill)
 {
 	uint32_t counter;
 	uint32_t blobsize = 0;
@@ -376,7 +374,8 @@ static uint32_t pak_instance_add_create_dir (struct pak_instance_t *self,
 	                dirdb_ref,
 	                0, /* refcount */
 	                1, /* is_archive */
-	                0  /* is_playlist */);
+	                0, /* is_playlist */
+	                self->archive_file->compression);
 
 	self->dirs[self->dir_fill]->owner = self;
 	self->dirs[self->dir_fill]->dir_parent = dir_parent;
@@ -453,7 +452,8 @@ static uint32_t pak_instance_add_file (struct pak_instance_t *self,
 	                 0, /* filename_override */
 	                 dirdb_ref,
 	                 0, /* refcount */
-	                 0  /* is_nodetect */);
+	                 0, /* is_nodetect */
+	                 COMPRESSION_ADD_STORE (self->archive_file->compression));
 
 	self->files[self->file_fill]->owner      = self;
 	self->files[self->file_fill]->dir_parent = dir_parent;
@@ -582,7 +582,8 @@ struct ocpdir_t *pak_check (const struct ocpdirdecompressor_t *self, struct ocpf
 	                file->dirdb_ref,
 	                0, /* refcount */
 	                1, /* is_archive */
-	                0  /* is_playlist */);
+	                0, /* is_playlist */
+	                file->compression);
 
 	file->parent->ref (file->parent);
 	iter->dirs[0]->owner = iter;
@@ -606,7 +607,7 @@ struct ocpdir_t *pak_check (const struct ocpdirdecompressor_t *self, struct ocpf
 	{
 		const char *filename = 0;
 		uint8_t *metadata = 0;
-		size_t metadatasize = 0;
+		uint32_t metadatasize = 0;
 
 		dirdbGetName_internalstr (iter->archive_file->dirdb_ref, &filename);
 		if (!adbMetaGet (filename, iter->archive_file->filesize (iter->archive_file), "PAK", &metadata, &metadatasize))
@@ -631,7 +632,7 @@ struct ocpdir_t *pak_check (const struct ocpdirdecompressor_t *self, struct ocpf
 		if (iter->file_fill)
 		{
 			uint8_t *metadata = 0;
-			size_t metadatasize = 0;
+			uint32_t metadatasize = 0;
 			const char *filename = 0;
 
 			pak_instance_encode_blob (iter, &metadata, &metadatasize);
@@ -1020,8 +1021,6 @@ static struct ocpfilehandle_t *pak_file_open (struct ocpfile_t *_self)
 	                       pak_filehandle_unref,
 	                       _self,
 	                       pak_filehandle_seek_set,
-	                       pak_filehandle_seek_cur,
-	                       pak_filehandle_seek_end,
 	                       pak_filehandle_getpos,
 	                       pak_filehandle_eof,
 	                       pak_filehandle_error,
@@ -1030,14 +1029,14 @@ static struct ocpfilehandle_t *pak_file_open (struct ocpfile_t *_self)
 	                       pak_filehandle_filesize,
 	                       pak_filehandle_filesize_ready,
 	                       0, /* filename_override */
-	                       dirdbRef (self->head.dirdb_ref, dirdb_use_filehandle));
+	                       dirdbRef (self->head.dirdb_ref, dirdb_use_filehandle),
+	                       1 /* refcount */
+	);
 
 	retval->file = self;
 
-	DEBUG_PRINT ("We just created a PAK handle, REF it\n");
-	retval->head.ref (&retval->head);
-	DEBUG_PRINT ("\n");
-
+	DEBUG_PRINT ("We just created a PAK handle, ref the source\n");
+	pak_instance_ref (self->owner);
 	pak_io_ref (self->owner);
 
 	return &retval->head;
@@ -1059,12 +1058,7 @@ static void pak_filehandle_ref (struct ocpfilehandle_t *_self)
 	struct pak_instance_filehandle_t *self = (struct pak_instance_filehandle_t *)_self;
 
 	DEBUG_PRINT ("pak_filehandle_ref (old count = %d)\n", self->head.refcount);
-	if (!self->head.refcount)
-	{
-		pak_instance_ref (self->file->owner);
-	}
 	self->head.refcount++;
-	DEBUG_PRINT ("\n");
 }
 
 static void pak_filehandle_unref (struct ocpfilehandle_t *_self)
@@ -1098,45 +1092,6 @@ static int pak_filehandle_seek_set (struct ocpfilehandle_t *_self, int64_t pos)
 	return 0;
 }
 
-static int pak_filehandle_seek_cur (struct ocpfilehandle_t *_self, int64_t pos)
-{
-	struct pak_instance_filehandle_t *self = (struct pak_instance_filehandle_t *)_self;
-
-	if (pos <= 0)
-	{
-		if (pos == INT64_MIN) return -1; /* we never have files this size */
-		if ((-pos) > self->filepos) return -1;
-		self->filepos += pos;
-	} else {
-		/* check for overflow */
-		if ((int64_t)(pos + self->filepos) < 0) return -1;
-
-		if ((pos + self->filepos) > self->file->filesize) return -1;
-		self->filepos += pos;
-	}
-
-	self->error = 0;
-
-	return 0;
-}
-
-static int pak_filehandle_seek_end (struct ocpfilehandle_t *_self, int64_t pos)
-{
-	struct pak_instance_filehandle_t *self = (struct pak_instance_filehandle_t *)_self;
-
-	if (pos > 0) return -1;
-
-	if (pos == INT64_MIN) return -1; /* we never have files this size */
-
-	if (pos < -(int64_t)(self->file->filesize)) return -1;
-
-	self->filepos = self->file->filesize + pos;
-
-	self->error = 0;
-
-	return 0;
-}
-
 static uint64_t pak_filehandle_getpos (struct ocpfilehandle_t *_self)
 {
 	struct pak_instance_filehandle_t *self = (struct pak_instance_filehandle_t *)_self;
@@ -1222,10 +1177,10 @@ static void pak_translate_complete (struct pak_instance_t *self)
 static void pak_translate (struct pak_instance_t *self, char *src, char **buffer, int *buffersize)
 {
 	char *temp;
-	size_t srclen;
+	uint32_t srclen;
 
 	char *dst = *buffer;
-	size_t dstlen = *buffersize;
+	uint32_t dstlen = *buffersize;
 
 	DEBUG_PRINT ("pak_translate %s =>", src);
 
@@ -1323,7 +1278,7 @@ static void pak_set_byuser_string (struct ocpdir_t *_self, const char *byuser)
 	/* update adbMeta */
 	{
 		uint8_t *metadata = 0;
-		size_t metadatasize = 0;
+		uint32_t metadatasize = 0;
 		const char *filename = 0;
 
 		pak_instance_encode_blob (self->owner, &metadata, &metadatasize);
