@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Code to provide Unix filesystem into the virtual drive FILE:
  *
@@ -106,10 +106,6 @@ static void unix_filehandle_unref (struct ocpfilehandle_t *_s);
 
 static int unix_filehandle_seek_set (struct ocpfilehandle_t *_s, int64_t pos);
 
-static int unix_filehandle_seek_cur (struct ocpfilehandle_t *_s, int64_t pos);
-
-static int unix_filehandle_seek_end (struct ocpfilehandle_t *_s, int64_t pos);
-
 static uint64_t unix_filehandle_getpos (struct ocpfilehandle_t *_s);
 
 static int unix_filehandle_eof (struct ocpfilehandle_t *_s);
@@ -422,7 +418,6 @@ static struct ocpfilehandle_t *unix_file_open (struct ocpfile_t *_s)
 		close (fd);
 		return 0;
 	}
-	r->head.refcount = 1;
 	r->fd = fd;
 	r->owner = s;
 	s->head.ref(&s->head);
@@ -433,17 +428,16 @@ static struct ocpfilehandle_t *unix_file_open (struct ocpfile_t *_s)
 		unix_filehandle_unref,
 		_s,
 		unix_filehandle_seek_set,
-		unix_filehandle_seek_cur,
-		unix_filehandle_seek_end,
 		unix_filehandle_getpos,
 		unix_filehandle_eof,
 		unix_filehandle_error,
 		unix_filehandle_read,
-	        0, /* ioctl */
+		0, /* ioctl */
 		unix_filehandle_filesize,
 		unix_filehandle_filesize_ready,
-	        0, /* filename_override */
-		dirdbRef (s->head.dirdb_ref, dirdb_use_filehandle)
+		0, /* filename_override */
+		dirdbRef (s->head.dirdb_ref, dirdb_use_filehandle),
+		1
 	);
 
 	return &r->head;
@@ -508,48 +502,6 @@ static int unix_filehandle_seek_set (struct ocpfilehandle_t *_s, int64_t pos)
 	return 0;
 }
 
-static int unix_filehandle_seek_cur (struct ocpfilehandle_t *_s, int64_t pos)
-{
-	struct unix_ocpfilehandle_t *s = (struct unix_ocpfilehandle_t *)_s;
-	off_t r;
-
-	r = lseek (s->fd, pos, SEEK_CUR);
-	if (r == (off_t) -1)
-	{
-		s->error = 1;
-		s->eof = 1;
-		return -1;
-	} else {
-		s->pos = r;
-	}
-
-	s->error = 0;
-	s->eof = (r >= s->owner->filesize);
-
-	return 0;
-}
-
-static int unix_filehandle_seek_end (struct ocpfilehandle_t *_s, int64_t pos)
-{
-	struct unix_ocpfilehandle_t *s = (struct unix_ocpfilehandle_t *)_s;
-	off_t r;
-
-	r = lseek (s->fd, pos, SEEK_END);
-	if (r == (off_t) -1)
-	{
-		s->error = 1;
-		s->eof = 1;
-		return -1;
-	} else {
-		s->pos = r;
-	}
-
-	s->error = 0;
-	s->eof = (r >= s->owner->filesize);
-
-	return 0;
-}
-
 static uint64_t unix_filehandle_getpos (struct ocpfilehandle_t *_s)
 {
 	struct unix_ocpfilehandle_t *s = (struct unix_ocpfilehandle_t *)_s;
@@ -649,7 +601,8 @@ static struct ocpdir_t *unix_dir_steal (struct ocpdir_t *parent, const uint32_t
 	               dirdb_node,
 	               1, /* refcount */
 	               0, /* is_archive */
-	               0  /* is_playlist */);
+	               0, /* is_playlist */
+	               COMPRESSION_NONE);
 
 	if (parent)
 	{
@@ -683,7 +636,8 @@ static struct ocpfile_t *unix_file_steal (struct ocpdir_t *parent, const uint32_
 		0, /* filename_override */
 		dirdb_node,
 		1, /* refcount */
-	        0  /* is_nodetect */
+	        0, /* is_nodetect */
+		COMPRESSION_NONE
 	);
 
 	parent->ref (parent);
