@@ -1,7 +1,7 @@
 #ifndef _FILESEL_FILESYSTEM_UNIX_H
 #define _FILESEL_FILESYSTEM_UNIX_H 1
 
-#ifndef __W32__
+#ifndef _WIN32
 
 struct ocpdir_t;
 struct ocpfile_t;
@@ -13,7 +13,7 @@ extern struct dmDrive *dmFile;
 
 struct ocpdir_t *file_unix_root (void);
 
-void filesystem_unix_init (void);
+int filesystem_unix_init (void);
 void filesystem_unix_done (void);
 
 #endif
