@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * GMDPlay - generic modular module player
  *
@@ -35,6 +35,7 @@
 #include "cpiface/cpiface.h"
 #include "dev/mcp.h"
 #include "gmdplay.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
 
 #define GMD_MAXPCHAN 32
@@ -214,7 +215,7 @@ static int quelen;
 static void readque (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int type,val1/*,val2*/;
-	int time = cpifaceSession->mcpGet (-1, mcpGTimer);
+	int time = cpifaceSession->mcpGet (cpifaceSession, -1, mcpGTimer);
 	while (1)
 	{
 		if (querpos==quewpos)
@@ -361,7 +362,7 @@ static uint8_t PlayNote (struct cpifaceSessionAPI_t *cpifaceSession, struct trac
 		{
 			if (t->phys!=-1)
 			{
-				cpifaceSession->mcpSet (t->phys, mcpCReset, 0);
+				cpifaceSession->mcpSet (cpifaceSession, t->phys, mcpCReset, 0);
 				pchan[t->phys]=-1;
 				t->phys=-1;
 			}
@@ -433,10 +434,10 @@ static void PlayGCommand (struct cpifaceSessionAPI_t *cpifaceSession, const uint
 				break;
 			case cmdSpeed:
 				speed=*cmd;
-				cpifaceSession->mcpSet(-1, mcpGSpeed, 256*2*speed/5);
+				cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, 256*2*speed/5);
 				break;
 			case cmdFineSpeed:
-				cpifaceSession->mcpSet(-1, mcpGSpeed, 256*2*(10*speed+*cmd)/50);
+				cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, 256*2*(10*speed+*cmd)/50);
 				break;
 			case cmdBreak:
 				if (brkpat==-1)
@@ -1131,7 +1132,7 @@ static void PlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 
 	if (donotloopmodule && looped)
 	{
-		cpifaceSession->mcpSet (-1, mcpMasterPause, 1);
+		cpifaceSession->mcpSet (cpifaceSession, -1, mcpMasterPause, 1);
 		return;
 	}
 
@@ -1139,10 +1140,10 @@ static void PlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 		return;
 
 	for (i=0; i<physchan; i++)
-		if (!cpifaceSession->mcpGet (i, mcpCStatus))
+		if (!cpifaceSession->mcpGet (cpifaceSession, i, mcpCStatus))
 			if (pchan[i]!=-1)
 			{
-				cpifaceSession->mcpSet (i, mcpCReset, 0);
+				cpifaceSession->mcpSet (cpifaceSession, i, mcpCReset, 0);
 				tdata[pchan[i]].phys=-1;
 				pchan[i]=-1;
 			}
@@ -1235,13 +1236,13 @@ static void PlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 				}
 				for (i=0; i<physchan; i++)
 				{
-					cpifaceSession->mcpSet (i, mcpCReset, 0);
+					cpifaceSession->mcpSet (cpifaceSession, i, mcpCReset, 0);
 					pchan[i]=-1;
 				}
 				tempo=6;
 				speed=125;
 				globalvol=0xFF;
-				cpifaceSession->mcpSet (-1, mcpGSpeed, 12800);
+				cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, 12800);
 			}
 			LoadPattern(currentpattern, currentrow);
 		}
@@ -1495,7 +1496,7 @@ static void PlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 
 		if (td->newinst!=-1)
 			if (td->phys!=-1)
-				cpifaceSession->mcpSet (td->phys, mcpCInstrument, td->newinst);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCInstrument, td->newinst);
 		if (td->newpos!=(uint_fast32_t)-1)
 		{
 			uint_fast32_t l;
@@ -1511,8 +1512,8 @@ static void PlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 					tdata[pchan[i]].phys=-1;
 				pchan[i]=td->num;
 				td->phys=i;
-				cpifaceSession->mcpSet (td->phys, mcpCReset, 0);
-				cpifaceSession->mcpSet (td->phys, mcpCInstrument, td->samp->handle);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCReset, 0);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCInstrument, td->samp->handle);
 			}
 			sm=&sampleinfos[td->samp->handle];
 			l=sm->length;
@@ -1525,46 +1526,55 @@ static void PlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 			if (td->newpos>=l)
 				td->newpos=l-16;
 
-			cpifaceSession->mcpSet (td->phys, mcpCPosition, td->newpos);
-			cpifaceSession->mcpSet (td->phys, mcpCLoop, 1);
-			cpifaceSession->mcpSet (td->phys, mcpCDirect, 0);
-			cpifaceSession->mcpSet (td->phys, mcpCStatus, 1);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPosition, td->newpos);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCLoop, 1);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCDirect, 0);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCStatus, 1);
 		}
 		if (td->newdir!=-1)
-			cpifaceSession->mcpSet (td->phys, mcpCDirect, td->newdir);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCDirect, td->newdir);
 		if (td->newloop!=-1)
-			cpifaceSession->mcpSet (td->phys, mcpCLoop, td->newloop);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCLoop, td->newloop);
 		if (td->phys!=-1)
 		{
 			if (td->stopchan)
-				cpifaceSession->mcpSet (td->phys, mcpCStatus, 0);
-			cpifaceSession->mcpSet (td->phys, mcpCVolume, (donotloopmodule&&looped)?0:vol);
-			cpifaceSession->mcpSet (td->phys, mcpCPanning, pan);
-			cpifaceSession->mcpSet (td->phys, mcpCPanY, td->pany);
-			cpifaceSession->mcpSet (td->phys, mcpCPanZ, td->panz);
-			cpifaceSession->mcpSet (td->phys, mcpCSurround, td->pansrnd);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCStatus, 0);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCVolume, (donotloopmodule&&looped)?0:vol);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPanning, pan);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPanY, td->pany);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPanZ, td->panz);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCSurround, td->pansrnd);
 			if (exponential)
-				cpifaceSession->mcpSet (td->phys, mcpCPitch, -checkpitche(td->finalpitch));
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPitch, -checkpitche(td->finalpitch));
 			else
-				cpifaceSession->mcpSet (td->phys, mcpCPitch6848, checkpitchh(td->finalpitch));
-			cpifaceSession->mcpSet(td->phys, mcpCMute, td->mute);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPitch6848, checkpitchh(td->finalpitch));
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCMute, td->mute);
 		}
 	}
 	readque (cpifaceSession);
-	cmdtime = cpifaceSession->mcpGet (-1, mcpGCmdTimer);
+	cmdtime = cpifaceSession->mcpGet (cpifaceSession, -1, mcpGCmdTimer);
 	putque(cmdtime, -1, (currentrow<<8)|(currentpattern<<16), 0);
 }
 
-char __attribute__ ((visibility ("internal"))) mpPlayModule(const struct gmdmodule *m, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL int mpPlayModule (const struct gmdmodule *m, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int i;
+
+	if (!cpifaceSession->mcpDevAPI)
+	{
+		return errPlay;
+	}
+
+
 	for (i=65; i<=128; i++)
 		sintab[i]=sintab[128-i];
 	for (i=129; i<256; i++)
 		sintab[i]=-sintab[256-i];
 
 	if (m->orders[0]==0xFFFF)
-		return 0;
+	{
+		return errFormStruc;
+	}
 
 	sampleinfos=m->samples;
 	modsampnum=m->modsampnum;
@@ -1617,32 +1627,32 @@ char __attribute__ ((visibility ("internal"))) mpPlayModule(const struct gmdmodu
 	quelen=100;
 	que=malloc(sizeof(int)*quelen*4);
 	if (!que)
-		return 0;
+	{
+		return errAllocMem;
+	}
 	querpos=0;
 	quewpos=0;
 
 	if (!cpifaceSession->mcpDevAPI->OpenPlayer (channels, PlayTick, file, cpifaceSession))
-		return 0;
+	{
+		return errPlay;
+	}
 
-	cpifaceSession->mcpAPI->Normalize (cpifaceSession, mcpNormalizeDefaultPlayW);
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayW);
 
 	physchan = cpifaceSession->PhysicalChannelCount;
 
-	return 1;
+	return errOk;
 }
 
-void __attribute__ ((visibility ("internal"))) mpStopModule (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL void mpStopModule (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	int i;
-	for (i=0; i<physchan; i++)
-	{
-		cpifaceSession->mcpSet (i, mcpCReset, 0);
-	}
-	cpifaceSession->mcpDevAPI->ClosePlayer ();
+	cpifaceSession->mcpDevAPI->ClosePlayer (cpifaceSession);
 	free(que);
+	que=0;
 }
 
-void __attribute__ ((visibility ("internal"))) mpGetChanInfo(uint8_t ch, struct chaninfo *ci)
+OCP_INTERNAL void mpGetChanInfo (uint8_t ch, struct chaninfo *ci)
 {
 	const struct trackdata *t=&tdata[ch];
 	ci->ins=0xFF;
@@ -1670,7 +1680,7 @@ void __attribute__ ((visibility ("internal"))) mpGetChanInfo(uint8_t ch, struct
 
 
 
-uint16_t __attribute__ ((visibility ("internal"))) mpGetRealNote(struct cpifaceSessionAPI_t *cpifaceSession, uint8_t ch)
+OCP_INTERNAL uint16_t mpGetRealNote (struct cpifaceSessionAPI_t *cpifaceSession, uint8_t ch)
 {
 	struct trackdata *td=&tdata[ch];
 	if (exponential)
@@ -1680,7 +1690,7 @@ uint16_t __attribute__ ((visibility ("internal"))) mpGetRealNote(struct cpifaceS
   /*    return td.nteval<<8; */
 }
 
-void __attribute__ ((visibility ("internal"))) mpGetGlobInfo(struct globinfo *gi)
+OCP_INTERNAL void mpGetGlobInfo (struct globinfo *gi)
 {
 	int i;
 
@@ -1698,19 +1708,19 @@ void __attribute__ ((visibility ("internal"))) mpGetGlobInfo(struct globinfo *gi
 			gi->globvolslide=globalvolslide[i];
 }
 
-void __attribute__ ((visibility ("internal"))) mpGetPosition(uint16_t *pat, uint8_t *row)
+OCP_INTERNAL void mpGetPosition (uint16_t *pat, uint8_t *row)
 {
 	*pat=currentpattern;
 	*row=currentrow;
 }
 
-int __attribute__ ((visibility ("internal"))) mpGetRealPos (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL int mpGetRealPos (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	readque (cpifaceSession);
 	return realpos;
 }
 
-void __attribute__ ((visibility ("internal"))) mpSetPosition (struct cpifaceSessionAPI_t *cpifaceSession, int16_t pat, int16_t row)
+OCP_INTERNAL void mpSetPosition (struct cpifaceSessionAPI_t *cpifaceSession, int16_t pat, int16_t row)
 {
 	unsigned int i;
 	if (row<0)
@@ -1753,7 +1763,7 @@ void __attribute__ ((visibility ("internal"))) mpSetPosition (struct cpifaceSess
 			lockpattern=pat;
 		for (i=0; i<physchan; i++)
 		{
-			cpifaceSession->mcpSet (i, mcpCReset, 0);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCReset, 0);
 			pchan[i]=-1;
 		}
 		for (i=0; i<channels; i++)
@@ -1768,17 +1778,17 @@ void __attribute__ ((visibility ("internal"))) mpSetPosition (struct cpifaceSess
 	currenttick=tempo;
 }
 
-char __attribute__ ((visibility ("internal"))) mpLooped(void)
+OCP_INTERNAL char mpLooped (void)
 {
 	return looped;
 }
 
-void __attribute__ ((visibility ("internal"))) mpSetLoop(uint8_t s)
+OCP_INTERNAL void mpSetLoop (uint8_t s)
 {
 	donotloopmodule=!s;
 }
 
-void __attribute__ ((visibility ("internal"))) mpLockPat(int st)
+OCP_INTERNAL void mpLockPat (int st)
 {
 	if (st)
 		lockpattern=currentpattern;
@@ -1786,7 +1796,7 @@ void __attribute__ ((visibility ("internal"))) mpLockPat(int st)
 		lockpattern=-1;
 }
 
-int __attribute__ ((visibility ("internal"))) mpGetChanSample (struct cpifaceSessionAPI_t *cpifaceSession, unsigned int ch, int16_t *buf, unsigned int len, uint32_t rate, int opt)
+OCP_INTERNAL int mpGetChanSample (struct cpifaceSessionAPI_t *cpifaceSession, unsigned int ch, int16_t *buf, unsigned int len, uint32_t rate, int opt)
 {
 	if (tdata[ch].phys==-1)
 	{
@@ -1796,22 +1806,22 @@ int __attribute__ ((visibility ("internal"))) mpGetChanSample (struct cpifaceSes
 	return cpifaceSession->mcpGetChanSample (cpifaceSession, tdata[ch].phys, buf, len, rate, opt);
 }
 
-void __attribute__ ((visibility ("internal"))) mpMute (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int mute)
+OCP_INTERNAL void mpMute (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int mute)
 {
 	cpifaceSession->MuteChannel[ch] = mute;
 	tdata[ch].mute=mute;
 	if (tdata[ch].phys!=-1)
-		cpifaceSession->mcpSet (tdata[ch].phys, mcpCMute, mute);
+		cpifaceSession->mcpSet (cpifaceSession, tdata[ch].phys, mcpCMute, mute);
 }
 
-int __attribute__ ((visibility ("internal"))) mpGetChanStatus (struct cpifaceSessionAPI_t *cpifaceSession, int ch)
+OCP_INTERNAL int mpGetChanStatus (struct cpifaceSessionAPI_t *cpifaceSession, int ch)
 {
 	if (tdata[ch].phys==-1)
 		return 0;
-	return cpifaceSession->mcpGet (tdata[ch].phys, mcpCStatus);
+	return cpifaceSession->mcpGet (cpifaceSession, tdata[ch].phys, mcpCStatus);
 }
 
-void __attribute__ ((visibility ("internal"))) mpGetRealVolume (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int *l, int *r)
+OCP_INTERNAL void mpGetRealVolume (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int *l, int *r)
 {
 	if (tdata[ch].phys==-1)
 	{
@@ -1821,7 +1831,7 @@ void __attribute__ ((visibility ("internal"))) mpGetRealVolume (struct cpifaceSe
 	cpifaceSession->mcpGetRealVolume (tdata[ch].phys, l, r);
 }
 
-int __attribute__ ((visibility ("internal"))) mpLoadSamples (struct cpifaceSessionAPI_t *cpifaceSession, struct gmdmodule *m)
+OCP_INTERNAL int mpLoadSamples (struct cpifaceSessionAPI_t *cpifaceSession, struct gmdmodule *m)
 {
-	return cpifaceSession->mcpDevAPI->LoadSamples (m->samples, m->sampnum);
+	return cpifaceSession->mcpDevAPI->LoadSamples (cpifaceSession, m->samples, m->sampnum);
 }
