@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2019-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2019-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * HVLPlay channel display routines
  *
@@ -88,7 +88,7 @@ static void drawvolbar (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bu
 static void drawlongvolbar (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *buf, int i, unsigned char st)
 {
 	int l,r;
-	mixGetRealVolume(i, &l, &r);
+	hvlGetChanVolume (cpifaceSession, i, &l, &r);
 	logvolbar(&l, &r);
 	l=(l+2)>>2;
 	r=(r+2)>>2;
@@ -614,7 +614,7 @@ static void drawchannel (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *b
 	}
 }
 
-void __attribute__ ((visibility ("internal"))) hvlChanSetup (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL void hvlChanSetup (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	cpifaceSession->UseChannels (cpifaceSession, drawchannel);
 }
