@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'20 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * ITPlayer - player for Impulse Tracker 2.xx modules
  *
@@ -82,6 +82,7 @@
 #include "cpiface/cpiface.h"
 #include "dev/mcp.h"
 #include "itplay.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
 
 
@@ -1432,38 +1433,38 @@ static void putchandata (struct cpifaceSessionAPI_t *cpifaceSession, struct itpl
 {
 	if (p->newsamp!=-1)
 	{
-		cpifaceSession->mcpSet (p->no, mcpCReset, 0);
-		cpifaceSession->mcpSet (p->no, mcpCInstrument, p->newsamp);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCReset, 0);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCInstrument, p->newsamp);
 		p->newsamp=-1;
 	}
 	if (p->newpos!=-1)
 	{
-		cpifaceSession->mcpSet (p->no, mcpCPosition, p->newpos);
-		cpifaceSession->mcpSet (p->no, mcpCLoop, 1);
-		cpifaceSession->mcpSet (p->no, mcpCDirect, 0);
-		cpifaceSession->mcpSet (p->no, mcpCStatus, 1);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCPosition, p->newpos);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCLoop, 1);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCDirect, 0);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCStatus, 1);
 		p->newpos=-1;
 		p->dead=0;
 	}
 	if (p->noteoff&&!p->looptype)
 	{
-		cpifaceSession->mcpSet (p->no, mcpCLoop, 2);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCLoop, 2);
 		p->looptype=1;
 	}
 	if (this->linear)
-		cpifaceSession->mcpSet (p->no, mcpCPitch, p->fpitch);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCPitch, p->fpitch);
 	else
-		cpifaceSession->mcpSet (p->no, mcpCPitch6848, -p->fpitch);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCPitch6848, -p->fpitch);
 
-	cpifaceSession->mcpSet (p->no, mcpCVolume, p->fvol);
-	cpifaceSession->mcpSet (p->no, mcpCPanning, p->fpan);
-	cpifaceSession->mcpSet (p->no, mcpCSurround, p->srnd);
-	cpifaceSession->mcpSet (p->no, mcpCMute, this->channels[p->lch].mute);
-	cpifaceSession->mcpSet (p->no, mcpCFilterFreq, p->fcutoff);
-	cpifaceSession->mcpSet (p->no, mcpCFilterRez, p->reso);
+	cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCVolume, p->fvol);
+	cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCPanning, p->fpan);
+	cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCSurround, p->srnd);
+	cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCMute, this->channels[p->lch].mute);
+	cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCFilterFreq, p->fcutoff);
+	cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCFilterRez, p->reso);
 }
 
-void __attribute__ ((visibility ("internal"))) mutechan(struct itplayer *this, int c, int m)
+OCP_INTERNAL void mutechan (struct itplayer *this, int c, int m)
 {
 	if ((c>=0)||(c<this->nchan))
 		this->channels[c].mute=m;
@@ -1533,12 +1534,12 @@ static void allocatechan(struct itplayer *this, struct it_logchan *c)
 
 static void getproctime (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this)
 {
-	this->proctime = cpifaceSession->mcpGet (-1, mcpGCmdTimer);
+	this->proctime = cpifaceSession->mcpGet (cpifaceSession, -1, mcpGCmdTimer);
 }
 
 static void putglobdata (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this)
 {
-	cpifaceSession->mcpSet (-1, mcpGSpeed, 256*2*this->tempo/5);
+	cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, 256*2*this->tempo/5);
 }
 
 static void putque(struct itplayer *this, int type, int val1, int val2)
@@ -1554,7 +1555,7 @@ static void putque(struct itplayer *this, int type, int val1, int val2)
 
 static int gettime (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	return cpifaceSession->mcpGet (-1, mcpGTimer);
+	return cpifaceSession->mcpGet (cpifaceSession, -1, mcpGTimer);
 }
 
 static void readque (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this)
@@ -1625,7 +1626,7 @@ static void readque (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer
 
 static void checkchan (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, struct it_physchan *p)
 {
-	if (!cpifaceSession->mcpGet (p->no, mcpCStatus))
+	if (!cpifaceSession->mcpGet (cpifaceSession, p->no, mcpCStatus))
 		p->dead=1;
 	if (p->dead&&(this->channels[p->lch].pch!=p))
 		p->notecut=1;
@@ -1634,7 +1635,7 @@ static void checkchan (struct cpifaceSessionAPI_t *cpifaceSession, struct itplay
 		if (this->channels[p->lch].pch==p)
 			this->channels[p->lch].pch=0;
 		p->lch=-1;
-		cpifaceSession->mcpSet(p->no, mcpCReset, 0);
+		cpifaceSession->mcpSet (cpifaceSession, p->no, mcpCReset, 0);
 		return;
 	}
 }
@@ -1645,7 +1646,7 @@ static void playtick (struct cpifaceSessionAPI_t *cpifaceSession, struct itplaye
 
 	if (this->looped&&this->noloop)
 	{
-		cpifaceSession->mcpSet (-1, mcpMasterPause, 1);
+		cpifaceSession->mcpSet (cpifaceSession, -1, mcpMasterPause, 1);
 		return;
 	}
 
@@ -1711,6 +1712,10 @@ static void playtick (struct cpifaceSessionAPI_t *cpifaceSession, struct itplaye
 					c->tremoroffcounter=0;
 				}
 			}
+			if (this->gotoord < this->curord && !this->manualgoto)
+			{
+				this->looped=1;
+			}
 			while (this->orders[this->gotoord]==0xFFFF)
 				this->gotoord++;
 			if (this->gotoord==this->endord)
@@ -1806,16 +1811,21 @@ static void playtick (struct cpifaceSessionAPI_t *cpifaceSession, struct itplaye
 	putque(this, quePos, -1, (this->curtick&0xFF)|(this->currow<<8)|(this->curord<<16));
 }
 
-int __attribute__ ((visibility ("internal"))) loadsamples (struct cpifaceSessionAPI_t *cpifaceSession, struct it_module *m)
+OCP_INTERNAL int loadsamples (struct cpifaceSessionAPI_t *cpifaceSession, struct it_module *m)
 {
-	return cpifaceSession->mcpDevAPI->LoadSamples(m->sampleinfos, m->nsampi);
+	return cpifaceSession->mcpDevAPI->LoadSamples (cpifaceSession, m->sampleinfos, m->nsampi);
 }
 
-int __attribute__ ((visibility ("internal"))) play (struct itplayer *this, const struct it_module *m, int ch, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL int itplay (struct itplayer *this, const struct it_module *m, int ch, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int i;
-	staticthis=this;
 
+	if (!cpifaceSession->mcpDevAPI)
+	{
+		return errPlay;
+	}
+
+	staticthis=this;
 	this->randseed=1;
 	this->patdelayrow=0;
 	this->patdelaytick=0;
@@ -1858,7 +1868,9 @@ int __attribute__ ((visibility ("internal"))) play (struct itplayer *this, const
 		this->curord++;
 
 	if (this->curord==this->nord)
-		return 0;
+	{
+		return errFormStruc;
+	}
 
 	this->channels=malloc(sizeof(struct it_logchan)*this->nchan);
 	this->pchannels=malloc(sizeof(struct it_physchan)*ch);
@@ -1881,7 +1893,7 @@ int __attribute__ ((visibility ("internal"))) play (struct itplayer *this, const
 			free(this->que);
 			this->que=NULL;
 		}
-		return 0;
+		return errFormStruc;
 	}
 	this->querpos=this->quewpos=0;
 	memset(this->channels, 0, sizeof(*this->channels)*this->nchan);
@@ -1904,18 +1916,20 @@ int __attribute__ ((visibility ("internal"))) play (struct itplayer *this, const
 	}
 
 	if (!cpifaceSession->mcpDevAPI->OpenPlayer(ch, playtickstatic, file, cpifaceSession))
-		return 0;
+	{
+		return errPlay;
+	}
 
-	cpifaceSession->mcpAPI->Normalize (cpifaceSession, mcpNormalizeDefaultPlayW);
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayW);
 
 	this->npchan = cpifaceSession->PhysicalChannelCount;
 
-	return 1;
+	return errOk;
 }
 
-void __attribute__ ((visibility ("internal"))) stop (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this)
+OCP_INTERNAL void itstop (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this)
 {
-	cpifaceSession->mcpDevAPI->ClosePlayer ();
+	cpifaceSession->mcpDevAPI->ClosePlayer (cpifaceSession);
 	if (this->channels)
 	{
 		free(this->channels);
@@ -1933,20 +1947,20 @@ void __attribute__ ((visibility ("internal"))) stop (struct cpifaceSessionAPI_t
 	}
 }
 
-int __attribute__ ((visibility ("internal"))) getpos(struct itplayer *this)
+OCP_INTERNAL int getpos (struct itplayer *this)
 {
 	if (this->manualgoto)
 		return (this->gotorow<<8)|(this->gotoord<<16);
 	return (this->curtick&0xFF)|(this->currow<<8)|(this->curord<<16);
 }
 
-int __attribute__ ((visibility ("internal"))) getrealpos (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this)
+OCP_INTERNAL int getrealpos (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this)
 {
 	readque (cpifaceSession, this);
 	return this->realpos;
 }
 
-int __attribute__ ((visibility ("internal"))) getchansample (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int16_t *buf, int len, uint32_t rate, int opt)
+OCP_INTERNAL int getchansample (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int16_t *buf, int len, uint32_t rate, int opt)
 {
 	int i,n;
 	unsigned int chn[64];
@@ -1958,7 +1972,7 @@ int __attribute__ ((visibility ("internal"))) getchansample (struct cpifaceSessi
 	return 1;
 }
 
-void __attribute__ ((visibility ("internal"))) itplayer_getrealvol (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int *l, int *r)
+OCP_INTERNAL void itplayer_getrealvol (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int *l, int *r)
 {
 	int i, voll, volr;
 
@@ -1973,7 +1987,7 @@ void __attribute__ ((visibility ("internal"))) itplayer_getrealvol (struct cpifa
 		}
 }
 
-void __attribute__ ((visibility ("internal"))) setpos(struct itplayer *this, int ord, int row)
+OCP_INTERNAL void setpos (struct itplayer *this, int ord, int row)
 {
 	int i;
 	if (this->curord!=ord)
@@ -1994,7 +2008,7 @@ void __attribute__ ((visibility ("internal"))) setpos(struct itplayer *this, int
 	this->realpos=(this->gotorow<<8)|(this->gotoord<<16);
 }
 
-int __attribute__ ((visibility ("internal"))) getdotsdata (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int pch, int *smp, int *note, int *voll, int *volr, int *sus)
+OCP_INTERNAL int getdotsdata (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int pch, int *smp, int *note, int *voll, int *volr, int *sus)
 {
 	struct it_physchan *p;
 	for (; pch<this->npchan; pch++)
@@ -2018,7 +2032,7 @@ int __attribute__ ((visibility ("internal"))) getdotsdata (struct cpifaceSession
 	return pch+1;
 }
 
-void __attribute__ ((visibility ("internal"))) getglobinfo (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int *tmp, int *bp, int *gv, int *gs)
+OCP_INTERNAL void getglobinfo (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int *tmp, int *bp, int *gv, int *gs)
 {
 	readque (cpifaceSession, this);
 	*tmp=this->realspeed;
@@ -2027,7 +2041,7 @@ void __attribute__ ((visibility ("internal"))) getglobinfo (struct cpifaceSessio
 	*gs=this->gvolslide?(this->gvolslide>0?ifxGVSUp:ifxGVSDown):0;
 }
 
-int __attribute__ ((visibility ("internal"))) getsync (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int *time)
+OCP_INTERNAL int getsync (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int *time)
 {
 	readque (cpifaceSession, this);
 	if ((ch<0)||(ch>=this->nchan))
@@ -2040,19 +2054,19 @@ int __attribute__ ((visibility ("internal"))) getsync (struct cpifaceSessionAPI_
 	}
 }
 /*
-int __attribute__ ((visibility ("internal"))) getticktime(struct itplayer *this)
+OCP_INTERNAL int getticktime (struct itplayer *this)
 {
 	readque(this);
 	return 65536*5/(2*this->realtempo);
 }
 
-int __attribute__ ((visibility ("internal"))) getrowtime(struct itplayer *this)
+OCP_INTERNAL int getrowtime (struct itplayer *this)
 {
 	readque(this);
 	return 65536*5*this->realspeed/(2*this->realtempo);
 }*/
 /*
-void __attribute__ ((visibility ("internal"))) setevpos(struct itplayer *this, int ch, int pos, int modtype, int mod)
+OCP_INTERNAL void setevpos (struct itplayer *this, int ch, int pos, int modtype, int mod)
 {
 	struct it_logchan *c;
 	if ((ch<0)||(ch>=this->nchan))
@@ -2067,7 +2081,7 @@ void __attribute__ ((visibility ("internal"))) setevpos(struct itplayer *this, i
 }
 */
 /*
-int __attribute__ ((visibility ("internal"))) getevpos(struct itplayer *this, int ch, int *time)
+OCP_INTERNAL int getevpos (struct itplayer *this, int ch, int *time)
 {
 	readque(this);
 	if ((ch<0)||(ch>=this->nchan))
@@ -2080,7 +2094,7 @@ int __attribute__ ((visibility ("internal"))) getevpos(struct itplayer *this, in
 }
 */
 /*
-int __attribute__ ((visibility ("internal"))) findevpos(struct itplayer *this, int pos, int *time)
+OCP_INTERNAL int findevpos (struct itplayer *this, int pos, int *time)
 {
 	int i;
 	readque(this);
@@ -2092,42 +2106,42 @@ int __attribute__ ((visibility ("internal"))) findevpos(struct itplayer *this, i
 }
 */
 
-void __attribute__ ((visibility ("internal"))) setloop(struct itplayer *this, int s)
+OCP_INTERNAL void setloop (struct itplayer *this, int s)
 {
 	this->noloop=!s;
 }
 
-int __attribute__ ((visibility ("internal"))) getloop(struct itplayer *this)
+OCP_INTERNAL int getloop (struct itplayer *this)
 {
 	return this->looped;
 }
 
-int __attribute__ ((visibility ("internal"))) chanactive (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int *lc)
+OCP_INTERNAL int chanactive (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int ch, int *lc)
 {
 	struct it_physchan *p=&this->pchannels[ch];
 	*lc=p->lch;
 	if (!(*lc>-1)&&p->smp&&p->fvol)
 		return 0; /* force mcpGet to be checked last - stian */
-	return cpifaceSession->mcpGet(ch, mcpCStatus);
+	return cpifaceSession->mcpGet(cpifaceSession, ch, mcpCStatus);
 }
 
-int __attribute__ ((visibility ("internal"))) lchanactive (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int lc)
+OCP_INTERNAL int lchanactive (struct cpifaceSessionAPI_t *cpifaceSession, struct itplayer *this, int lc)
 {
 	struct it_physchan *p=this->channels[lc].pch;
 	if (!p)
 		return 0; /* avoid strange crashes on some archs - stian */
 	if (!(p->smp&&p->fvol))
 		return 0; /* force mcpGet to be checked last - stian */
-	return cpifaceSession->mcpGet (p->no, mcpCStatus);
+	return cpifaceSession->mcpGet (cpifaceSession, p->no, mcpCStatus);
 }
 
-int __attribute__ ((visibility ("internal"))) getchanins(struct itplayer *this, int ch)
+OCP_INTERNAL int getchanins (struct itplayer *this, int ch)
 {
 	struct it_physchan *p=&this->pchannels[ch];
 	return p->inst->handle+1;
 }
 
-int __attribute__ ((visibility ("internal"))) getchansamp(struct itplayer *this, int ch)
+OCP_INTERNAL int getchansamp (struct itplayer *this, int ch)
 {
 	struct it_physchan *p=&this->pchannels[ch];
 	if (!p->smp)
@@ -2136,7 +2150,7 @@ int __attribute__ ((visibility ("internal"))) getchansamp(struct itplayer *this,
 }
 
 
-void __attribute__ ((visibility ("internal"))) getchaninfo(struct itplayer *this, uint8_t ch, struct it_chaninfo *ci)
+OCP_INTERNAL void getchaninfo (struct itplayer *this, uint8_t ch, struct it_chaninfo *ci)
 {
 	const struct it_logchan *t=&this->channels[ch];
 	if (t->pch)
@@ -2160,7 +2174,7 @@ void __attribute__ ((visibility ("internal"))) getchaninfo(struct itplayer *this
 		memset(ci, 0, sizeof(*ci));
 }
 
-int __attribute__ ((visibility ("internal"))) getchanalloc(struct itplayer *this, uint8_t ch)
+OCP_INTERNAL int getchanalloc (struct itplayer *this, uint8_t ch)
 {
 	int num=0;
 	int i;
