@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'20 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * ITPlay track/pattern display code
  *
@@ -534,7 +534,7 @@ static struct cpitrakdisplaystruct xmtrakdisplay=
 	it_getins, it_getvol, it_getpan, it_getfx, it_getgcmd
 };
 
-void __attribute__ ((visibility ("internal"))) itTrkSetup (struct cpifaceSessionAPI_t *cpifaceSession, const struct it_module *mod)
+OCP_INTERNAL void itTrkSetup (struct cpifaceSessionAPI_t *cpifaceSession, const struct it_module *mod)
 {
 	plPatterns=mod->patterns;
 	plOrders=mod->orders;
