@@ -101,7 +101,8 @@
    GNU application programs can use a third alternative mode in which
    they can distinguish the relative order of options and other arguments.  */
 
-#include "getopt.h"
+/* If this file is compiled, it is because our target system is lacking getopt.h */
+#include "mygetopt.h"
 
 /* For communication from `getopt' to the caller.
    When `getopt' finds an option that takes an argument,
