@@ -163,9 +163,9 @@ void CldsPlayer::gettrackdata(unsigned char pattern, void (*callback)(void *arg,
     unsigned short   patnum = positions[pattern * 9 + channel].patnum;
     unsigned char transpose = positions[pattern * 9 + channel].transpose;
     int crow=0;
-    int packpos = 0;
+    unsigned int packpos = 0;
     while (crow < pattlen) {
-      unsigned short comword, chan;
+      unsigned short comword;
       unsigned char  comhi, comlo;
 
       unsigned char note = 0;
@@ -268,7 +268,7 @@ void CldsPlayer::gettrackdata(unsigned char pattern, void (*callback)(void *arg,
               high = (comhi + transp) << 4;
             }
 
-            if(chandelay[chan]) {
+            if(chandelay[channel]) {
               // command = TrackedCmdDelay;
               // param = chandelay[chan];
             }
