@@ -547,7 +547,7 @@ public:
 #endif
 
 	unsigned char       GetTrackFor(unsigned long Ordr);
-        void                GetTrackData(unsigned char pattern, void (*callback)(void *arg, unsigned char row, unsigned char channel, unsigned char note, CPlayer::TrackedCmds command, unsigned char inst, unsigned char volume, unsigned char param), void *arg);
+	void                GetTrackData(unsigned char pattern, void (*callback)(void *arg, unsigned char row, unsigned char channel, unsigned char note, CPlayer::TrackedCmds command, unsigned char inst, unsigned char volume, unsigned char param), void *arg);
 
 private:
 	bool                UnpackNote(uint8_t *&s, uint8_t &last_instrument);
@@ -674,7 +674,7 @@ void RADPlayer::Init(const void *tune, void(*opl3)(void *, uint16_t, uint8_t), v
 		return;
 	}
 	Version = ver >> 4;
-	UseOPL3 = ver >= 2;
+	UseOPL3 = Version >= 2;
 
 	// The OPL3 call-back
 	OPL3 = opl3;
