@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Glue logic for using libsidplayfp playback engine
  *
@@ -78,15 +78,15 @@ namespace libsidplayfp
 	{
 		switch (model)
 		{
-			case MOS656X::MOS6567R56A:
+			case MOS656X::model_t::MOS6567R56A:
 				return "MOS6567R56A NTSC-M (old)";
-			case MOS656X::MOS6567R8:
+			case MOS656X::model_t::MOS6567R8:
 				return "MOS6567R8 NTSC-M";
-			case MOS656X::MOS6569:
+			case MOS656X::model_t::MOS6569:
 				return "MOS6569 PAL-B";
-			case MOS656X::MOS6572:
+			case MOS656X::model_t::MOS6572:
 				return "MOS6572 PAL-N";
-			case MOS656X::MOS6573:
+			case MOS656X::model_t::MOS6573:
 				return "MOS6573 PAL-M";
 			default:
 				return "MOS65xx ??";
@@ -245,6 +245,16 @@ namespace libsidplayfp
 			//fprintf (stderr, "filtercurve6581=%lf\n", m_filter.filterCurve6581);
 		}
 
+		const char *range6581 = configAPI->GetProfileString("libsidplayfp", "filterrange6581", "0.5");
+		m_filter.filterRange6581 = strtod(range6581, &endptr);
+		if ((*endptr != 0) || (range6581 == endptr))
+		{
+			fprintf (stderr, "[libsidplayfp]\n  filterrange6581=invalid... defaulting to 0.5\n");
+			m_filter.filterRange6581=0.5;
+		} else {
+			//fprintf (stderr, "filterrange6581=%lf\n", m_filter.filterRange6581);
+		}
+
 		const char *curve8580 = configAPI->GetProfileString("libsidplayfp", "filtercurve8580", "0.5");
 		m_filter.filterCurve8580 = strtod(curve8580, &endptr);
 		if ((*endptr != 0) || (curve8580 == endptr))
@@ -255,6 +265,21 @@ namespace libsidplayfp
 			//fprintf (stderr, "filtercurve8580=%lf\n", m_filter.filterCurve8580);
 		}
 
+		const char *CWS = configAPI->GetProfileString("libsidplayfp", "combinedwaveforms", "Average");
+		if (!strcasecmp(CWS, "Weak"))
+		{
+			m_filter.combinedWaveforms = SidConfig::WEAK;
+		} else if (!strcasecmp(CWS, "Strong"))
+		{
+			m_filter.combinedWaveforms = SidConfig::STRONG;
+		} else if (!strcasecmp(CWS, "Average"))
+		{
+			m_filter.combinedWaveforms = SidConfig::AVERAGE;
+		} else {
+			fprintf (stderr, "[libsidplayfp]\n  combinedwaveforms=invalid... defaulting to Average\n");
+			m_filter.combinedWaveforms = SidConfig::AVERAGE;
+		}
+
 		m_engCfg.digiBoost = configAPI->GetProfileBool("libsidplayfp", "digiboost", 0, 0);
 		//fprintf (stderr, "digiboost=%d\n", m_engCfg.digiBoost);
 
@@ -287,8 +312,6 @@ namespace libsidplayfp
 		dirdbAPI->Unref (  basic_ref, dirdb_use_file);
 		dirdbAPI->Unref (chargen_ref, dirdb_use_file);
 
-		dirdbAPI->Unref ( dirdb_base, dirdb_use_dir);
-
 		sidplayer.setKernal ( kernalRom);
 		sidplayer.setBasic  (  basicRom);
 		sidplayer.setChargen(chargenRom);
@@ -376,14 +399,10 @@ namespace libsidplayfp
 					goto createSidEmu_error;
 				}
 
-				if (m_filter.filterCurve6581)
-				{
-					rs->filter6581Curve(m_filter.filterCurve6581);
-				}
-				if (m_filter.filterCurve8580)
-				{
-					rs->filter8580Curve(m_filter.filterCurve8580);
-				}
+				rs->filter6581Curve(m_filter.filterCurve6581);
+				rs->filter6581Range(m_filter.filterRange6581);
+				rs->filter8580Curve(m_filter.filterCurve8580);
+				rs->combinedWaveformsStrength(m_filter.combinedWaveforms);
 			} else {
 				ReSIDBuilder *rs = new ReSIDBuilder( "ReSID" );
 
@@ -410,7 +429,9 @@ namespace libsidplayfp
 			return false;
 		} else {
 			/* set up SID filter. HardSID just ignores call with def. */
-			m_engCfg.sidEmulation->filter(m_filter.enabled);
+			sidplayer.filter(0, m_filter.enabled);
+			sidplayer.filter(1, m_filter.enabled);
+			sidplayer.filter(2, m_filter.enabled);
 		}
 
 		return true;
@@ -519,6 +540,75 @@ namespace libsidplayfp
 		sidplayer.mute (chan / 3, chan % 3, mute);
 	}
 
+	void ConsolePlayer::SetFilter(bool enable)
+	{
+		const SidConfig &config = sidplayer.config ();
+		sidplayer.filter(0, enable);
+		sidplayer.filter(1, enable);
+		sidplayer.filter(2, enable);
+	}
+
+	void ConsolePlayer::SetBias(double bias)
+	{
+		if (bias >  500.0) bias =  500.0;
+		if (bias < -500.0) bias = -500.0;
+
+		ReSIDBuilder *rs = dynamic_cast<ReSIDBuilder *>(m_engCfg.sidEmulation);
+		if (rs)
+		{
+			rs->bias (bias);
+		}
+	}
+
+	void ConsolePlayer::SetFilterCurve6581 (double v)
+	{
+		if (v > 1.0) v = 1.0;
+		if (v < 0.0) v = 0.0;
+
+		ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
+		if (rs)
+		{
+			rs->filter6581Curve (v);
+		}
+	}
+
+	void ConsolePlayer::SetFilterRange6581 (double v)
+	{
+		if (v > 1.0) v = 1.0;
+		if (v < 0.0) v = 0.0;
+
+		ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
+		if (rs)
+		{
+			rs->filter6581Range (v);
+		}
+	}
+
+	void ConsolePlayer::SetFilterCurve8580 (double v)
+	{
+		if (v > 1.0) v = 1.0;
+		if (v < 0.0) v = 0.0;
+
+		ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
+		if (rs)
+		{
+			rs->filter8580Curve (v);
+		}
+	}
+
+	void ConsolePlayer::SetCombinedWaveformsStrength (int CWS)
+	{
+		if (CWS < 0) CWS = 0;
+		if (CWS > 2) CWS = 2;
+		ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
+		if (rs)
+		{
+			if (CWS == 0) rs->combinedWaveformsStrength (SidConfig::AVERAGE);
+			if (CWS == 1) rs->combinedWaveformsStrength (SidConfig::WEAK);
+			if (CWS == 2) rs->combinedWaveformsStrength (SidConfig::STRONG);
+		}
+	}
+
 	const SidTuneInfo::clock_t ConsolePlayer::getTuneInfoClockSpeed(void)
 	{
 		const SidTuneInfo *tuneInfo = m_tune.getInfo();
