@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * XMPlay - Module player for XM/MOD and affiliate formats
  *
@@ -276,7 +276,7 @@ static void ReadQue (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int type,val1,val2,t;
 	int i;
-	int time = cpifaceSession->mcpGet(-1, mcpGTimer);
+	int time = cpifaceSession->mcpGet (cpifaceSession, -1, mcpGTimer);
 	while (1)
 	{
 		if (querpos==quewpos)
@@ -513,17 +513,17 @@ static void xmpPlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 
 	if ((!looping) && looped)
 	{
-		cpifaceSession->mcpSet (-1, mcpMasterPause, 1);
+		cpifaceSession->mcpSet (cpifaceSession, -1, mcpMasterPause, 1);
 		return;
 	}
 
 	if (firstspeed)
 	{
-		cpifaceSession->mcpSet (-1, mcpGSpeed, firstspeed);
+		cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, firstspeed);
 		firstspeed=0;
 	}
 
-	cmdtime = cpifaceSession->mcpGet (-1, mcpGCmdTimer);
+	cmdtime = cpifaceSession->mcpGet (cpifaceSession, -1, mcpGCmdTimer);
 	ReadQue (cpifaceSession);
 
 	tick0=0;
@@ -795,7 +795,7 @@ static void xmpPlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 					if (procdat>=0x20)
 					{
 						curbpm=procdat;
-						cpifaceSession->mcpSet (-1, mcpGSpeed, 256*2*curbpm/5);
+						cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, 256*2*curbpm/5);
 						putque(queTempo, -1, curbpm);
 					} else {
 						curtempo=procdat;
@@ -848,8 +848,9 @@ static void xmpPlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 						if (ch->cursamp->panenv<nenv)
 							if (ch->chPanEnvPos>envelopes[ch->cursamp->panenv].len)
 								ch->chPanEnvPos=envelopes[ch->cursamp->panenv].len;
-					} else
+					} else {
 						fprintf(stderr, __FILE__ " CmdEnvPos ch->cursamp not set\n");
+					}
 					break;
 				case xmpCmdPanSlide:
 					if (procdat)
@@ -1274,7 +1275,7 @@ static void xmpPlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 
 		if (!ch->cursamp)
 		{
-			cpifaceSession->mcpSet (i, mcpCStatus, 0);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCStatus, 0);
 			continue;
 		}
 
@@ -1355,55 +1356,55 @@ static void xmpPlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 		}
 
 		if (ch->nextstop)
-			cpifaceSession->mcpSet (i, mcpCStatus, 0);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCStatus, 0);
 		if (ch->nextsamp!=(unsigned)-1)
-			cpifaceSession->mcpSet( i, mcpCInstrument, ch->nextsamp);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCInstrument, ch->nextsamp);
 		if (ch->nextpos!=(unsigned)-1)
 		{
-			cpifaceSession->mcpSet (i, mcpCPosition, ch->nextpos);
-			cpifaceSession->mcpSet (i, mcpCLoop, 1);
-			cpifaceSession->mcpSet (i, mcpCDirect, 0);
-			cpifaceSession->mcpSet (i, mcpCStatus, 1);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCPosition, ch->nextpos);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCLoop, 1);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCDirect, 0);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCStatus, 1);
 		}
 		if (linearfreq)
-			cpifaceSession->mcpSet (i, mcpCPitch, -ch->chFinalPitch);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCPitch, -ch->chFinalPitch);
 		else
-			cpifaceSession->mcpSet (i, mcpCPitch6848, ch->chFinalPitch);
-		cpifaceSession->mcpSet (i, mcpCVolume, (looping||!looped)?vol:0);
-		cpifaceSession->mcpSet (i, mcpCPanning, pan);
-		cpifaceSession->mcpSet (i, mcpCMute, mutech[i]);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCPitch6848, ch->chFinalPitch);
+		cpifaceSession->mcpSet (cpifaceSession, i, mcpCVolume, (looping||!looped)?vol:0);
+		cpifaceSession->mcpSet (cpifaceSession, i, mcpCPanning, pan);
+		cpifaceSession->mcpSet (cpifaceSession, i, mcpCMute, mutech[i]);
 	}
 	putque(quePos, -1, curtick|(curord<<16)|(currow<<8));
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetRealPos (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL int xmpGetRealPos (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	ReadQue (cpifaceSession);
 	return realpos;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpChanActive (struct cpifaceSessionAPI_t *cpifaceSession, int ch)
+OCP_INTERNAL int xmpChanActive (struct cpifaceSessionAPI_t *cpifaceSession, int ch)
 {
-	return cpifaceSession->mcpGet(ch, mcpCStatus)&&channels[ch].cursamp&&channels[ch].chVol&&channels[ch].chFadeVol;
+	return cpifaceSession->mcpGet (cpifaceSession, ch, mcpCStatus)&&channels[ch].cursamp&&channels[ch].chVol&&channels[ch].chFadeVol;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetChanIns(int ch)
+OCP_INTERNAL int xmpGetChanIns (int ch)
 {
 	return channels[ch].chCurIns;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetChanSamp(int ch)
+OCP_INTERNAL int xmpGetChanSamp (int ch)
 {
 	if (!channels[ch].cursamp)
 		return 0xFFFF;
 	return channels[ch].cursamp-samples;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetDotsData (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int *smp, int *frq, int *voll, int *volr, int *sus)
+OCP_INTERNAL int xmpGetDotsData (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int *smp, int *frq, int *voll, int *volr, int *sus)
 {
 	struct channel *c;
 
-	if (!cpifaceSession->mcpGet (ch, mcpCStatus))
+	if (!cpifaceSession->mcpGet (cpifaceSession, ch, mcpCStatus))
 		return 0;
 	c=&channels[ch];
 	if (!c->cursamp||!c->chVol||!c->chFadeVol)
@@ -1418,17 +1419,17 @@ int __attribute__ ((visibility ("internal"))) xmpGetDotsData (struct cpifaceSess
 	return 1;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpGetRealVolume (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int *voll, int *volr)
+OCP_INTERNAL void xmpGetRealVolume (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int *voll, int *volr)
 {
 	cpifaceSession->mcpGetRealVolume (ch, voll, volr);
 }
 
-uint16_t __attribute__ ((visibility ("internal"))) xmpGetPos(void)
+OCP_INTERNAL uint16_t xmpGetPos (void)
 {
 	return (curord<<8)|currow;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpSetPos (struct cpifaceSessionAPI_t *cpifaceSession, int ord, int row)
+OCP_INTERNAL void xmpSetPos (struct cpifaceSessionAPI_t *cpifaceSession, int ord, int row)
 {
 	int i;
 
@@ -1455,7 +1456,7 @@ void __attribute__ ((visibility ("internal"))) xmpSetPos (struct cpifaceSessionA
 			row=0;
 	}
 	for (i=0; i<nchan; i++)
-		cpifaceSession->mcpSet (i, mcpCReset, 0);
+		cpifaceSession->mcpSet (cpifaceSession, i, mcpCReset, 0);
 	jumptoord=ord;
 	jumptorow=row;
 	curtick=curtempo;
@@ -1467,28 +1468,28 @@ void __attribute__ ((visibility ("internal"))) xmpSetPos (struct cpifaceSessionA
 	realpos=(curord<<16)|(currow<<8);
 }
 
-void __attribute__ ((visibility ("internal"))) xmpMute (struct cpifaceSessionAPI_t *cpifaceSession, int i, int m)
+OCP_INTERNAL void xmpMute (struct cpifaceSessionAPI_t *cpifaceSession, int i, int m)
 {
 	cpifaceSession->MuteChannel[i] = m;
 	mutech[i]=m;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpLoop(void)
+OCP_INTERNAL int xmpLoop (void)
 {
 	return looped;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpSetLoop(int x)
+OCP_INTERNAL void xmpSetLoop (int x)
 {
 	looping=x;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpLoadSamples (struct cpifaceSessionAPI_t *cpifaceSession, struct xmodule *m)
+OCP_INTERNAL int xmpLoadSamples (struct cpifaceSessionAPI_t *cpifaceSession, struct xmodule *m)
 {
-	return cpifaceSession->mcpDevAPI->LoadSamples (m->sampleinfos, m->nsampi);
+	return cpifaceSession->mcpDevAPI->LoadSamples (cpifaceSession, m->sampleinfos, m->nsampi);
 }
 
-int __attribute__ ((visibility ("internal"))) xmpPlayModule (struct xmodule *m, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL int xmpPlayModule (struct xmodule *m, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int i;
 
@@ -1532,7 +1533,9 @@ int __attribute__ ((visibility ("internal"))) xmpPlayModule (struct xmodule *m,
 	quelen=100;
 	que=malloc(sizeof(int)*quelen*4);
 	if (!que)
-		return 0;
+	{
+		return errAllocMem;
+	}
 	querpos=0;
 	quewpos=0;
 
@@ -1541,33 +1544,36 @@ int __attribute__ ((visibility ("internal"))) xmpPlayModule (struct xmodule *m,
 	realspeed=m->initempo;
 	firstspeed=256*2*curbpm/5;
 	if (!cpifaceSession->mcpDevAPI->OpenPlayer(nchan, xmpPlayTick, file, cpifaceSession))
-		return 0;
+	{
+		return errPlay;
+	}
 
-	cpifaceSession->mcpAPI->Normalize (cpifaceSession, mcpNormalizeDefaultPlayW);
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayW);
 
 	if (nchan != cpifaceSession->PhysicalChannelCount)
 	{
-		cpifaceSession->mcpDevAPI->ClosePlayer ();
-		return 0;
+		cpifaceSession->mcpDevAPI->ClosePlayer (cpifaceSession);
+		return errFormStruc;
 	}
 
-	return 1;
+	return errOk;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpStopModule (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL void xmpStopModule (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	cpifaceSession->mcpDevAPI->ClosePlayer ();
+	cpifaceSession->mcpDevAPI->ClosePlayer (cpifaceSession);
 	free(que);
+	que=0;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpGetGlobInfo(int *tmp, int *bpm, int *gvol)
+OCP_INTERNAL void xmpGetGlobInfo (int *tmp, int *bpm, int *gvol)
 {
 	*tmp=realspeed;
 	*bpm=realtempo;
 	*gvol=realgvol;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpGetChanInfo(unsigned char ch, struct xmpchaninfo *ci)
+OCP_INTERNAL void xmpGetChanInfo (unsigned char ch, struct xmpchaninfo *ci)
 {
 	const struct channel *t=&channels[ch];
 	ci->note=t->curnote+11;
@@ -1585,7 +1591,7 @@ void __attribute__ ((visibility ("internal"))) xmpGetChanInfo(unsigned char ch,
 	ci->fx=t->fx;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpGetGlobInfo2(struct xmpglobinfo *gi)
+OCP_INTERNAL void xmpGetGlobInfo2 (struct xmpglobinfo *gi)
 {
 	gi->globvol=globalvol;
 	gi->globvolslide=globalfx;
